@@ -2,7 +2,7 @@
  * VampirTrace
  * http://www.tu-dresden.de/zih/vampirtrace
  *
- * Copyright (c) 2005-2012, ZIH, TU Dresden, Federal Republic of Germany
+ * Copyright (c) 2005-2013, ZIH, TU Dresden, Federal Republic of Germany
  *
  * Copyright (c) 1998-2005, Forschungszentrum Juelich, Juelich Supercomputing
  *                          Centre, Federal Republic of Germany
@@ -13,14 +13,10 @@
 #ifndef _VT_UNIFY_TKFAC_SCOPE_H_
 #define _VT_UNIFY_TKFAC_SCOPE_H_
 
-#include "config.h"
-
 #include "vt_unify.h"
 
 #include <algorithm>
 #include <iostream>
-#include <map>
-#include <set>
 
 //
 // TokenFactoryScopeI interface class
