@@ -1,10 +1,12 @@
-/* -*- Mode: C; c-basic-offset:4 ; -*- */
+/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil  -*- */
 /*
- * Copyright (c) 2009      The University of Tennessee and The University
+ * Copyright (c) 2009-2010 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2009      Oak Ridge National Labs.  All rights reserved.
- * Copyright (c) 2010      Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2010-2012 Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2013      Los Alamos National Security, LLC. All rights
+ *                         reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -24,67 +26,172 @@
 #define OMPI_DATATYPE_INTERNAL_H
 
 #include "opal/datatype/opal_datatype_internal.h"
+#include "ompi/class/ompi_free_list.h"
 
 /*
  * This is the OMPI-layered numbering of ALL supported MPI types
  * (derived from the old DT_ names).
- * NOTE: These numbers have to match the d_f_to_c_index.
  */
-#define OMPI_DATATYPE_MPI_NULL                    0x00
-#define OMPI_DATATYPE_MPI_LB                      0x00
-#define OMPI_DATATYPE_MPI_UB                      0x01
-#define OMPI_DATATYPE_MPI_CHAR                    0x02
-#define OMPI_DATATYPE_MPI_SIGNED_CHAR             0x03   /* Changed order */
-#define OMPI_DATATYPE_MPI_UNSIGNED_CHAR           0x04   /* Changed order */
-#define OMPI_DATATYPE_MPI_BYTE                    0x05
-#define OMPI_DATATYPE_MPI_SHORT                   0x06
-#define OMPI_DATATYPE_MPI_UNSIGNED_SHORT          0x07
-#define OMPI_DATATYPE_MPI_INT                     0x08
-#define OMPI_DATATYPE_MPI_UNSIGNED_INT            0x09
-#define OMPI_DATATYPE_MPI_LONG                    0x0A
-#define OMPI_DATATYPE_MPI_UNSIGNED_LONG           0x0B
-#define OMPI_DATATYPE_MPI_LONG_LONG               0x0C   /* Was LONG_LONG_INT */
-#define OMPI_DATATYPE_MPI_UNSIGNED_LONG_LONG      0x0D
-#define OMPI_DATATYPE_MPI_FLOAT                   0x0E
-#define OMPI_DATATYPE_MPI_DOUBLE                  0x0F
-#define OMPI_DATATYPE_MPI_LONG_DOUBLE             0x10
-#define OMPI_DATATYPE_MPI_COMPLEX8                0x11
-#define OMPI_DATATYPE_MPI_COMPLEX16               0x12
-#define OMPI_DATATYPE_MPI_COMPLEX32               0x13
-#define OMPI_DATATYPE_MPI_WCHAR                   0x14
-#define OMPI_DATATYPE_MPI_PACKED                  0x15
+#define OMPI_DATATYPE_MPI_EMPTY                   0x00
+#define OMPI_DATATYPE_MPI_INT8_T                  0x01
+#define OMPI_DATATYPE_MPI_UINT8_T                 0x02
+#define OMPI_DATATYPE_MPI_INT16_T                 0x03
+#define OMPI_DATATYPE_MPI_UINT16_T                0x04
+#define OMPI_DATATYPE_MPI_INT32_T                 0x05
+#define OMPI_DATATYPE_MPI_UINT32_T                0x06
+#define OMPI_DATATYPE_MPI_INT64_T                 0x07
+#define OMPI_DATATYPE_MPI_UINT64_T                0x08
+#define OMPI_DATATYPE_MPI_FLOAT                   0x09
+#define OMPI_DATATYPE_MPI_DOUBLE                  0x0A
+#define OMPI_DATATYPE_MPI_LONG_DOUBLE             0x0B
+#define OMPI_DATATYPE_MPI_COMPLEX8                0x0C
+#define OMPI_DATATYPE_MPI_COMPLEX16               0x0D
+#define OMPI_DATATYPE_MPI_COMPLEX32               0x0E
+#define OMPI_DATATYPE_MPI_WCHAR                   0x0F
+#define OMPI_DATATYPE_MPI_PACKED                  0x10
 
-#define OMPI_DATATYPE_MPI_BOOL                    0x16   /* Was CXX_BOOL */
+#define OMPI_DATATYPE_MPI_BOOL                    0x11
 
-#define OMPI_DATATYPE_MPI_LOGICAL                 0x17
-#define OMPI_DATATYPE_MPI_CHARACTER               0x18   /* Changed */
-#define OMPI_DATATYPE_MPI_INTEGER                 0x19
-#define OMPI_DATATYPE_MPI_REAL                    0x1A
-#define OMPI_DATATYPE_MPI_DOUBLE_PRECISION        0x1B   /* Was DBLPREC */
+#define OMPI_DATATYPE_MPI_LOGICAL                 0x12
+#define OMPI_DATATYPE_MPI_CHARACTER               0x13
+#define OMPI_DATATYPE_MPI_INTEGER                 0x14
+#define OMPI_DATATYPE_MPI_REAL                    0x15
+#define OMPI_DATATYPE_MPI_DOUBLE_PRECISION        0x16
 
 /*
  * Derived datatypes supposely contiguous
  */
-#define OMPI_DATATYPE_MPI_COMPLEX                 0x1C   /* Was COMPLEX_FLOAT */
-#define OMPI_DATATYPE_MPI_DOUBLE_COMPLEX          0x1D   /* Was COMPLEX_DOUBLE */
-#define OMPI_DATATYPE_MPI_LONG_DOUBLE_COMPLEX     0x1E   /* Was COMPLEX_LONG_DOUBLE */
-#define OMPI_DATATYPE_MPI_2INT                    0x1F
-#define OMPI_DATATYPE_MPI_2INTEGER                0x20
-#define OMPI_DATATYPE_MPI_2REAL                   0x21
-#define OMPI_DATATYPE_MPI_2DBLPREC                0x22
-#define OMPI_DATATYPE_MPI_2COMPLEX                0x23
-#define OMPI_DATATYPE_MPI_2DOUBLE_COMPLEX         0x24
+#define OMPI_DATATYPE_MPI_COMPLEX                 0x17   /* Was COMPLEX_FLOAT */
+#define OMPI_DATATYPE_MPI_DOUBLE_COMPLEX          0x18   /* Was COMPLEX_DOUBLE */
+#define OMPI_DATATYPE_MPI_LONG_DOUBLE_COMPLEX     0x19   /* Was COMPLEX_LONG_DOUBLE */
+#define OMPI_DATATYPE_MPI_2INT                    0x1A
+#define OMPI_DATATYPE_MPI_2INTEGER                0x1B
+#define OMPI_DATATYPE_MPI_2REAL                   0x1C
+#define OMPI_DATATYPE_MPI_2DBLPREC                0x1D
+#define OMPI_DATATYPE_MPI_2COMPLEX                0x1E
+#define OMPI_DATATYPE_MPI_2DOUBLE_COMPLEX         0x1F
 /*
  * Derived datatypes which will definitively be non contiguous on some architectures.
  */
-#define OMPI_DATATYPE_MPI_FLOAT_INT               0x25
-#define OMPI_DATATYPE_MPI_DOUBLE_INT              0x26
-#define OMPI_DATATYPE_MPI_LONG_DOUBLE_INT         0x27
-#define OMPI_DATATYPE_MPI_LONG_INT                0x28
-#define OMPI_DATATYPE_MPI_SHORT_INT               0x29
-#define OMPI_DATATYPE_MPI_UNAVAILABLE             0x2A
+#define OMPI_DATATYPE_MPI_FLOAT_INT               0x20
+#define OMPI_DATATYPE_MPI_DOUBLE_INT              0x21
+#define OMPI_DATATYPE_MPI_LONG_DOUBLE_INT         0x22
+#define OMPI_DATATYPE_MPI_LONG_INT                0x23
+#define OMPI_DATATYPE_MPI_SHORT_INT               0x24
+/*
+ * Datatypes from the MPI 2.2 standard
+ */
+#define OMPI_DATATYPE_MPI_AINT                    0x25
+#define OMPI_DATATYPE_MPI_OFFSET                  0x26
+#define OMPI_DATATYPE_MPI_C_BOOL                  0x27
+#define OMPI_DATATYPE_MPI_C_COMPLEX               0x28
+#define OMPI_DATATYPE_MPI_C_FLOAT_COMPLEX         0x29
+#define OMPI_DATATYPE_MPI_C_DOUBLE_COMPLEX        0x2A
+#define OMPI_DATATYPE_MPI_C_LONG_DOUBLE_COMPLEX   0x2B
 
-#define OMPI_DATATYPE_MPI_MAX_PREDEFINED          0x2B
+#define OMPI_DATATYPE_MPI_LB                      0x2C
+#define OMPI_DATATYPE_MPI_UB                      0x2D
+
+/*
+ * Datatypes from the MPI 3.0 standard
+ */
+#define OMPI_DATATYPE_MPI_COUNT                   0x2E
+
+/* This should __ALWAYS__ stay last  */
+#define OMPI_DATATYPE_MPI_UNAVAILABLE             0x2F
+
+
+#define OMPI_DATATYPE_MPI_MAX_PREDEFINED          (OMPI_DATATYPE_MPI_UNAVAILABLE+1)
+
+/*
+ * Ensure we can support the predefined datatypes.
+ */
+#if OMPI_DATATYPE_MAX_PREDEFINED < OMPI_DATATYPE_MPI_UNAVAILABLE
+#error OMPI_DATATYPE_MAX_PREDEFINED should be updated to the value of OMPI_DATATYPE_MPI_UNAVAILABLE
+#endif
+
+/*
+ * Mapped types. The following types have basic equivalents in OPAL. Instead
+ * of being redefined as independent types, they will be made synonyms to
+ * the most basic type.
+ */
+#if SIZEOF_CHAR == 1
+#define OMPI_DATATYPE_MPI_CHAR                    OMPI_DATATYPE_MPI_INT8_T
+#define OMPI_DATATYPE_MPI_SIGNED_CHAR             OMPI_DATATYPE_MPI_INT8_T
+#define OMPI_DATATYPE_MPI_UNSIGNED_CHAR           OMPI_DATATYPE_MPI_UINT8_T
+#define OMPI_DATATYPE_MPI_BYTE                    OMPI_DATATYPE_MPI_UINT8_T
+#elif SIZEOF_CHAR == 2
+#define OMPI_DATATYPE_MPI_CHAR                    OMPI_DATATYPE_MPI_INT16_T
+#define OMPI_DATATYPE_MPI_SIGNED_CHAR             OMPI_DATATYPE_MPI_INT16_T
+#define OMPI_DATATYPE_MPI_UNSIGNED_CHAR           OMPI_DATATYPE_MPI_UINT16_T
+#define OMPI_DATATYPE_MPI_BYTE                    OMPI_DATATYPE_MPI_UINT16_T
+#elif SIZEOF_CHAR == 4
+#define OMPI_DATATYPE_MPI_CHAR                    OMPI_DATATYPE_MPI_INT32_T
+#define OMPI_DATATYPE_MPI_SIGNED_CHAR             OMPI_DATATYPE_MPI_INT32_T
+#define OMPI_DATATYPE_MPI_UNSIGNED_CHAR           OMPI_DATATYPE_MPI_UINT32_T
+#define OMPI_DATATYPE_MPI_BYTE                    OMPI_DATATYPE_MPI_UINT32_T
+#elif SIZEOF_CHAR == 8
+#define OMPI_DATATYPE_MPI_CHAR                    OMPI_DATATYPE_MPI_INT64_T
+#define OMPI_DATATYPE_MPI_SIGNED_CHAR             OMPI_DATATYPE_MPI_INT64_T
+#define OMPI_DATATYPE_MPI_UNSIGNED_CHAR           OMPI_DATATYPE_MPI_UINT64_T
+#define OMPI_DATATYPE_MPI_BYTE                    OMPI_DATATYPE_MPI_UINT64_T
+#endif
+
+#if SIZEOF_SHORT == 1
+#define OMPI_DATATYPE_MPI_SHORT                   OMPI_DATATYPE_MPI_INT8_T
+#define OMPI_DATATYPE_MPI_UNSIGNED_SHORT          OMPI_DATATYPE_MPI_UINT8_T
+#elif SIZEOF_SHORT == 2
+#define OMPI_DATATYPE_MPI_SHORT                   OMPI_DATATYPE_MPI_INT16_T
+#define OMPI_DATATYPE_MPI_UNSIGNED_SHORT          OMPI_DATATYPE_MPI_UINT16_T
+#elif SIZEOF_SHORT == 4
+#define OMPI_DATATYPE_MPI_SHORT                   OMPI_DATATYPE_MPI_INT32_T
+#define OMPI_DATATYPE_MPI_UNSIGNED_SHORT          OMPI_DATATYPE_MPI_UINT32_T
+#elif SIZEOF_SHORT == 8
+#define OMPI_DATATYPE_MPI_SHORT                   OMPI_DATATYPE_MPI_INT64_T
+#define OMPI_DATATYPE_MPI_UNSIGNED_SHORT          OMPI_DATATYPE_MPI_UINT64_T
+#endif
+
+#if SIZEOF_INT == 1
+#define OMPI_DATATYPE_MPI_INT                     OMPI_DATATYPE_MPI_INT8_T
+#define OMPI_DATATYPE_MPI_UNSIGNED_INT            OMPI_DATATYPE_MPI_UINT8_T
+#elif SIZEOF_INT == 2
+#define OMPI_DATATYPE_MPI_INT                     OMPI_DATATYPE_MPI_INT16_T
+#define OMPI_DATATYPE_MPI_UNSIGNED_INT            OMPI_DATATYPE_MPI_UINT16_T
+#elif SIZEOF_INT == 4
+#define OMPI_DATATYPE_MPI_INT                     OMPI_DATATYPE_MPI_INT32_T
+#define OMPI_DATATYPE_MPI_UNSIGNED_INT            OMPI_DATATYPE_MPI_UINT32_T
+#elif SIZEOF_INT == 8
+#define OMPI_DATATYPE_MPI_INT                     OMPI_DATATYPE_MPI_INT64_T
+#define OMPI_DATATYPE_MPI_UNSIGNED_INT            OMPI_DATATYPE_MPI_UINT64_T
+#endif
+
+#if SIZEOF_LONG == 1
+#define OMPI_DATATYPE_MPI_LONG                    OMPI_DATATYPE_MPI_INT8_T
+#define OMPI_DATATYPE_MPI_UNSIGNED_LONG           OMPI_DATATYPE_MPI_UINT8_T
+#elif SIZEOF_LONG == 2
+#define OMPI_DATATYPE_MPI_LONG                    OMPI_DATATYPE_MPI_INT16_T
+#define OMPI_DATATYPE_MPI_UNSIGNED_LONG           OMPI_DATATYPE_MPI_UINT16_T
+#elif SIZEOF_LONG == 4
+#define OMPI_DATATYPE_MPI_LONG                    OMPI_DATATYPE_MPI_INT32_T
+#define OMPI_DATATYPE_MPI_UNSIGNED_LONG           OMPI_DATATYPE_MPI_UINT32_T
+#elif SIZEOF_LONG == 8
+#define OMPI_DATATYPE_MPI_LONG                    OMPI_DATATYPE_MPI_INT64_T
+#define OMPI_DATATYPE_MPI_UNSIGNED_LONG           OMPI_DATATYPE_MPI_UINT64_T
+#endif
+
+#if SIZEOF_LONG_LONG == 1
+#define OMPI_DATATYPE_MPI_LONG_LONG               OMPI_DATATYPE_MPI_INT8_T
+#define OMPI_DATATYPE_MPI_UNSIGNED_LONG_LONG      OMPI_DATATYPE_MPI_UINT8_T
+#elif SIZEOF_LONG_LONG == 2
+#define OMPI_DATATYPE_MPI_LONG_LONG               OMPI_DATATYPE_MPI_INT16_T
+#define OMPI_DATATYPE_MPI_UNSIGNED_LONG_LONG      OMPI_DATATYPE_MPI_UINT16_T
+#elif SIZEOF_LONG_LONG == 4
+#define OMPI_DATATYPE_MPI_LONG_LONG               OMPI_DATATYPE_MPI_INT32_T
+#define OMPI_DATATYPE_MPI_UNSIGNED_LONG_LONG      OMPI_DATATYPE_MPI_UINT32_T
+#elif SIZEOF_LONG_LONG == 8
+#define OMPI_DATATYPE_MPI_LONG_LONG               OMPI_DATATYPE_MPI_INT64_T
+#define OMPI_DATATYPE_MPI_UNSIGNED_LONG_LONG      OMPI_DATATYPE_MPI_UINT64_T
+#endif
 
 /*
  * Optional Fortran datatypes, these map to representable types
@@ -95,56 +202,56 @@
 /* LOGICAL */
 #if OMPI_SIZEOF_FORTRAN_LOGICAL1 == OMPI_SIZEOF_FORTRAN_LOGICAL
 #  define OMPI_DATATYPE_MPI_LOGICAL1              OMPI_DATATYPE_MPI_LOGICAL
-#elif OMPI_SIZEOF_FORTRAN_LOGICAL1 == SIZEOF_CHAR
-#  define OMPI_DATATYPE_MPI_LOGICAL1              OMPI_DATATYPE_MPI_CHAR
-#elif OMPI_SIZEOF_FORTRAN_LOGICAL1 == SIZEOF_SHORT
-#  define OMPI_DATATYPE_MPI_LOGICAL1              OMPI_DATATYPE_MPI_SHORT
-#elif OMPI_SIZEOF_FORTRAN_LOGICAL1 == SIZEOF_INT
-#  define OMPI_DATATYPE_MPI_LOGICAL1              OMPI_DATATYPE_MPI_INT
-#elif OMPI_SIZEOF_FORTRAN_LOGICAL1 == SIZEOF_LONG
-#  define OMPI_DATATYPE_MPI_LOGICAL1              OMPI_DATATYPE_MPI_LONG
+#elif OMPI_SIZEOF_FORTRAN_LOGICAL1 == 1
+#  define OMPI_DATATYPE_MPI_LOGICAL1              OMPI_DATATYPE_MPI_INT8_T
+#elif OMPI_SIZEOF_FORTRAN_LOGICAL1 == 2
+#  define OMPI_DATATYPE_MPI_LOGICAL1              OMPI_DATATYPE_MPI_INT16_T
+#elif OMPI_SIZEOF_FORTRAN_LOGICAL1 == 4
+#  define OMPI_DATATYPE_MPI_LOGICAL1              OMPI_DATATYPE_MPI_INT32_T
+#elif OMPI_SIZEOF_FORTRAN_LOGICAL1 == 8
+#  define OMPI_DATATYPE_MPI_LOGICAL1              OMPI_DATATYPE_MPI_INT64_T
 #else
 #  define OMPI_DATATYPE_MPI_LOGICAL1              OMPI_DATATYPE_MPI_UNAVAILABLE
 #endif
 
 #if OMPI_SIZEOF_FORTRAN_LOGICAL2 == OMPI_SIZEOF_FORTRAN_LOGICAL
 #  define OMPI_DATATYPE_MPI_LOGICAL2              OMPI_DATATYPE_MPI_LOGICAL
-#elif OMPI_SIZEOF_FORTRAN_LOGICAL2 == SIZEOF_CHAR
-#  define OMPI_DATATYPE_MPI_LOGICAL2              OMPI_DATATYPE_MPI_CHAR
-#elif OMPI_SIZEOF_FORTRAN_LOGICAL2 == SIZEOF_SHORT
-#  define OMPI_DATATYPE_MPI_LOGICAL2              OMPI_DATATYPE_MPI_SHORT
-#elif OMPI_SIZEOF_FORTRAN_LOGICAL2 == SIZEOF_INT
-#  define OMPI_DATATYPE_MPI_LOGICAL2              OMPI_DATATYPE_MPI_INT
-#elif OMPI_SIZEOF_FORTRAN_LOGICAL2 == SIZEOF_LONG
-#  define OMPI_DATATYPE_MPI_LOGICAL2              OMPI_DATATYPE_MPI_LONG
+#elif OMPI_SIZEOF_FORTRAN_LOGICAL2 == 1
+#  define OMPI_DATATYPE_MPI_LOGICAL2              OMPI_DATATYPE_MPI_INT8_T
+#elif OMPI_SIZEOF_FORTRAN_LOGICAL2 == 2
+#  define OMPI_DATATYPE_MPI_LOGICAL2              OMPI_DATATYPE_MPI_INT16_T
+#elif OMPI_SIZEOF_FORTRAN_LOGICAL2 == 4
+#  define OMPI_DATATYPE_MPI_LOGICAL2              OMPI_DATATYPE_MPI_INT32_T
+#elif OMPI_SIZEOF_FORTRAN_LOGICAL2 == 8
+#  define OMPI_DATATYPE_MPI_LOGICAL2              OMPI_DATATYPE_MPI_INT64_T
 #else
 #  define OMPI_DATATYPE_MPI_LOGICAL2              OMPI_DATATYPE_MPI_UNAVAILABLE
 #endif
 
 #if OMPI_SIZEOF_FORTRAN_LOGICAL4 == OMPI_SIZEOF_FORTRAN_LOGICAL
 #  define OMPI_DATATYPE_MPI_LOGICAL4              OMPI_DATATYPE_MPI_LOGICAL
-#elif OMPI_SIZEOF_FORTRAN_LOGICAL4 == SIZEOF_CHAR
-#  define OMPI_DATATYPE_MPI_LOGICAL4              OMPI_DATATYPE_MPI_CHAR
-#elif OMPI_SIZEOF_FORTRAN_LOGICAL4 == SIZEOF_SHORT
-#  define OMPI_DATATYPE_MPI_LOGICAL4              OMPI_DATATYPE_MPI_SHORT
-#elif OMPI_SIZEOF_FORTRAN_LOGICAL4 == SIZEOF_INT
-#  define OMPI_DATATYPE_MPI_LOGICAL4              OMPI_DATATYPE_MPI_INT
-#elif OMPI_SIZEOF_FORTRAN_LOGICAL4 == SIZEOF_LONG
-#  define OMPI_DATATYPE_MPI_LOGICAL4              OMPI_DATATYPE_MPI_LONG
+#elif OMPI_SIZEOF_FORTRAN_LOGICAL4 == 1
+#  define OMPI_DATATYPE_MPI_LOGICAL4              OMPI_DATATYPE_MPI_INT8_T
+#elif OMPI_SIZEOF_FORTRAN_LOGICAL4 == 2
+#  define OMPI_DATATYPE_MPI_LOGICAL4              OMPI_DATATYPE_MPI_INT16_T
+#elif OMPI_SIZEOF_FORTRAN_LOGICAL4 == 4
+#  define OMPI_DATATYPE_MPI_LOGICAL4              OMPI_DATATYPE_MPI_INT32_T
+#elif OMPI_SIZEOF_FORTRAN_LOGICAL4 == 8
+#  define OMPI_DATATYPE_MPI_LOGICAL4              OMPI_DATATYPE_MPI_INT64_T
 #else
 #  define OMPI_DATATYPE_MPI_LOGICAL4              OMPI_DATATYPE_MPI_UNAVAILABLE
 #endif
 
 #if OMPI_SIZEOF_FORTRAN_LOGICAL8 == OMPI_SIZEOF_FORTRAN_LOGICAL
 #  define OMPI_DATATYPE_MPI_LOGICAL8              OMPI_DATATYPE_MPI_LOGICAL
-#elif OMPI_SIZEOF_FORTRAN_LOGICAL8 == SIZEOF_CHAR
-#  define OMPI_DATATYPE_MPI_LOGICAL8              OMPI_DATATYPE_MPI_CHAR
-#elif OMPI_SIZEOF_FORTRAN_LOGICAL8 == SIZEOF_SHORT
-#  define OMPI_DATATYPE_MPI_LOGICAL8              OMPI_DATATYPE_MPI_SHORT
-#elif OMPI_SIZEOF_FORTRAN_LOGICAL8 == SIZEOF_INT
-#  define OMPI_DATATYPE_MPI_LOGICAL8              OMPI_DATATYPE_MPI_INT
-#elif OMPI_SIZEOF_FORTRAN_LOGICAL8 == SIZEOF_LONG
-#  define OMPI_DATATYPE_MPI_LOGICAL8              OMPI_DATATYPE_MPI_LONG
+#elif OMPI_SIZEOF_FORTRAN_LOGICAL8 == 1
+#  define OMPI_DATATYPE_MPI_LOGICAL8              OMPI_DATATYPE_MPI_INT8_T
+#elif OMPI_SIZEOF_FORTRAN_LOGICAL8 == 2
+#  define OMPI_DATATYPE_MPI_LOGICAL8              OMPI_DATATYPE_MPI_INT16_T
+#elif OMPI_SIZEOF_FORTRAN_LOGICAL8 == 4
+#  define OMPI_DATATYPE_MPI_LOGICAL8              OMPI_DATATYPE_MPI_INT32_T
+#elif OMPI_SIZEOF_FORTRAN_LOGICAL8 == 8
+#  define OMPI_DATATYPE_MPI_LOGICAL8              OMPI_DATATYPE_MPI_INT64_T
 #else
 #  define OMPI_DATATYPE_MPI_LOGICAL8              OMPI_DATATYPE_MPI_UNAVAILABLE
 #endif
@@ -152,70 +259,70 @@
 /* INTEGER */
 #if OMPI_SIZEOF_FORTRAN_INTEGER1 == OMPI_SIZEOF_FORTRAN_INTEGER
 #  define OMPI_DATATYPE_MPI_INTEGER1              OMPI_DATATYPE_MPI_INTEGER
-#elif OMPI_SIZEOF_FORTRAN_INTEGER1 == SIZEOF_CHAR
-#  define OMPI_DATATYPE_MPI_INTEGER1              OMPI_DATATYPE_MPI_CHAR
-#elif OMPI_SIZEOF_FORTRAN_INTEGER1 == SIZEOF_SHORT
-#  define OMPI_DATATYPE_MPI_INTEGER1              OMPI_DATATYPE_MPI_SHORT
-#elif OMPI_SIZEOF_FORTRAN_INTEGER1 == SIZEOF_INT
-#  define OMPI_DATATYPE_MPI_INTEGER1              OMPI_DATATYPE_MPI_INT
-#elif OMPI_SIZEOF_FORTRAN_INTEGER1 == SIZEOF_LONG
-#  define OMPI_DATATYPE_MPI_INTEGER1              OMPI_DATATYPE_MPI_LONG
+#elif OMPI_SIZEOF_FORTRAN_INTEGER1 == 1
+#  define OMPI_DATATYPE_MPI_INTEGER1              OMPI_DATATYPE_MPI_INT8_T
+#elif OMPI_SIZEOF_FORTRAN_INTEGER1 == 2
+#  define OMPI_DATATYPE_MPI_INTEGER1              OMPI_DATATYPE_MPI_INT16_T
+#elif OMPI_SIZEOF_FORTRAN_INTEGER1 == 4
+#  define OMPI_DATATYPE_MPI_INTEGER1              OMPI_DATATYPE_MPI_INT32_T
+#elif OMPI_SIZEOF_FORTRAN_INTEGER1 == 8
+#  define OMPI_DATATYPE_MPI_INTEGER1              OMPI_DATATYPE_MPI_INT64_T
 #else
 #  define OMPI_DATATYPE_MPI_INTEGER1              OMPI_DATATYPE_MPI_UNAVAILABLE
 #endif
 
 #if OMPI_SIZEOF_FORTRAN_INTEGER2 == OMPI_SIZEOF_FORTRAN_INTEGER
 #  define OMPI_DATATYPE_MPI_INTEGER2              OMPI_DATATYPE_MPI_INTEGER
-#elif OMPI_SIZEOF_FORTRAN_INTEGER2 == SIZEOF_CHAR
-#  define OMPI_DATATYPE_MPI_INTEGER2              OMPI_DATATYPE_MPI_CHAR
-#elif OMPI_SIZEOF_FORTRAN_INTEGER2 == SIZEOF_SHORT
-#  define OMPI_DATATYPE_MPI_INTEGER2              OMPI_DATATYPE_MPI_SHORT
-#elif OMPI_SIZEOF_FORTRAN_INTEGER2 == SIZEOF_INT
-#  define OMPI_DATATYPE_MPI_INTEGER2              OMPI_DATATYPE_MPI_INT
-#elif OMPI_SIZEOF_FORTRAN_INTEGER2 == SIZEOF_LONG
-#  define OMPI_DATATYPE_MPI_INTEGER2              OMPI_DATATYPE_MPI_LONG
+#elif OMPI_SIZEOF_FORTRAN_INTEGER2 == 1
+#  define OMPI_DATATYPE_MPI_INTEGER2              OMPI_DATATYPE_MPI_INT8_T
+#elif OMPI_SIZEOF_FORTRAN_INTEGER2 == 2
+#  define OMPI_DATATYPE_MPI_INTEGER2              OMPI_DATATYPE_MPI_INT16_T
+#elif OMPI_SIZEOF_FORTRAN_INTEGER2 == 4
+#  define OMPI_DATATYPE_MPI_INTEGER2              OMPI_DATATYPE_MPI_INT32_T
+#elif OMPI_SIZEOF_FORTRAN_INTEGER2 == 8
+#  define OMPI_DATATYPE_MPI_INTEGER2              OMPI_DATATYPE_MPI_INT64_T
 #else
 #  define OMPI_DATATYPE_MPI_INTEGER2              OMPI_DATATYPE_MPI_UNAVAILABLE
 #endif
 
 #if OMPI_SIZEOF_FORTRAN_INTEGER4 == OMPI_SIZEOF_FORTRAN_INTEGER
 #  define OMPI_DATATYPE_MPI_INTEGER4              OMPI_DATATYPE_MPI_INTEGER
-#elif OMPI_SIZEOF_FORTRAN_INTEGER4 == SIZEOF_CHAR
-#  define OMPI_DATATYPE_MPI_INTEGER4              OMPI_DATATYPE_MPI_CHAR
-#elif OMPI_SIZEOF_FORTRAN_INTEGER4 == SIZEOF_SHORT
-#  define OMPI_DATATYPE_MPI_INTEGER4              OMPI_DATATYPE_MPI_SHORT
-#elif OMPI_SIZEOF_FORTRAN_INTEGER4 == SIZEOF_INT
-#  define OMPI_DATATYPE_MPI_INTEGER4              OMPI_DATATYPE_MPI_INT
-#elif OMPI_SIZEOF_FORTRAN_INTEGER4 == SIZEOF_LONG
-#  define OMPI_DATATYPE_MPI_INTEGER4              OMPI_DATATYPE_MPI_LONG
+#elif OMPI_SIZEOF_FORTRAN_INTEGER4 == 1
+#  define OMPI_DATATYPE_MPI_INTEGER4              OMPI_DATATYPE_MPI_INT8_T
+#elif OMPI_SIZEOF_FORTRAN_INTEGER4 == 2
+#  define OMPI_DATATYPE_MPI_INTEGER4              OMPI_DATATYPE_MPI_INT16_T
+#elif OMPI_SIZEOF_FORTRAN_INTEGER4 == 4
+#  define OMPI_DATATYPE_MPI_INTEGER4              OMPI_DATATYPE_MPI_INT32_T
+#elif OMPI_SIZEOF_FORTRAN_INTEGER4 == 8
+#  define OMPI_DATATYPE_MPI_INTEGER4              OMPI_DATATYPE_MPI_INT64_T
 #else
 #  define OMPI_DATATYPE_MPI_INTEGER4              OMPI_DATATYPE_MPI_UNAVAILABLE
 #endif
 
 #if OMPI_SIZEOF_FORTRAN_INTEGER8 == OMPI_SIZEOF_FORTRAN_INTEGER
 #  define OMPI_DATATYPE_MPI_INTEGER8              OMPI_DATATYPE_MPI_INTEGER
-#elif OMPI_SIZEOF_FORTRAN_INTEGER8 == SIZEOF_CHAR
-#  define OMPI_DATATYPE_MPI_INTEGER8              OMPI_DATATYPE_MPI_CHAR
-#elif OMPI_SIZEOF_FORTRAN_INTEGER8 == SIZEOF_SHORT
-#  define OMPI_DATATYPE_MPI_INTEGER8              OMPI_DATATYPE_MPI_SHORT
-#elif OMPI_SIZEOF_FORTRAN_INTEGER8 == SIZEOF_INT
-#  define OMPI_DATATYPE_MPI_INTEGER8              OMPI_DATATYPE_MPI_INT
-#elif OMPI_SIZEOF_FORTRAN_INTEGER8 == SIZEOF_LONG
-#  define OMPI_DATATYPE_MPI_INTEGER8              OMPI_DATATYPE_MPI_LONG
+#elif OMPI_SIZEOF_FORTRAN_INTEGER8 == 1
+#  define OMPI_DATATYPE_MPI_INTEGER8              OMPI_DATATYPE_MPI_INT8_T
+#elif OMPI_SIZEOF_FORTRAN_INTEGER8 == 2
+#  define OMPI_DATATYPE_MPI_INTEGER8              OMPI_DATATYPE_MPI_INT16_T
+#elif OMPI_SIZEOF_FORTRAN_INTEGER8 == 4
+#  define OMPI_DATATYPE_MPI_INTEGER8              OMPI_DATATYPE_MPI_INT32_T
+#elif OMPI_SIZEOF_FORTRAN_INTEGER8 == 8
+#  define OMPI_DATATYPE_MPI_INTEGER8              OMPI_DATATYPE_MPI_INT64_T
 #else
 #  define OMPI_DATATYPE_MPI_INTEGER8              OMPI_DATATYPE_MPI_UNAVAILABLE
 #endif
 
 #if OMPI_SIZEOF_FORTRAN_INTEGER16 == OMPI_SIZEOF_FORTRAN_INTEGER
 #  define OMPI_DATATYPE_MPI_INTEGER16             OMPI_DATATYPE_MPI_INTEGER
-#elif OMPI_SIZEOF_FORTRAN_INTEGER16 == SIZEOF_CHAR
-#  define OMPI_DATATYPE_MPI_INTEGER16             OMPI_DATATYPE_MPI_CHAR
-#elif OMPI_SIZEOF_FORTRAN_INTEGER16 == SIZEOF_SHORT
-#  define OMPI_DATATYPE_MPI_INTEGER16             OMPI_DATATYPE_MPI_SHORT
-#elif OMPI_SIZEOF_FORTRAN_INTEGER16 == SIZEOF_INT
-#  define OMPI_DATATYPE_MPI_INTEGER16             OMPI_DATATYPE_MPI_INT
-#elif OMPI_SIZEOF_FORTRAN_INTEGER16 == SIZEOF_LONG
-#  define OMPI_DATATYPE_MPI_INTEGER16             OMPI_DATATYPE_MPI_LONG
+#elif OMPI_SIZEOF_FORTRAN_INTEGER16 == 1
+#  define OMPI_DATATYPE_MPI_INTEGER16             OMPI_DATATYPE_MPI_INT8_T
+#elif OMPI_SIZEOF_FORTRAN_INTEGER16 == 2
+#  define OMPI_DATATYPE_MPI_INTEGER16             OMPI_DATATYPE_MPI_INT16_T
+#elif OMPI_SIZEOF_FORTRAN_INTEGER16 == 4
+#  define OMPI_DATATYPE_MPI_INTEGER16             OMPI_DATATYPE_MPI_INT32_T
+#elif OMPI_SIZEOF_FORTRAN_INTEGER16 == 8
+#  define OMPI_DATATYPE_MPI_INTEGER16             OMPI_DATATYPE_MPI_INT64_T
 #else
 #  define OMPI_DATATYPE_MPI_INTEGER16             OMPI_DATATYPE_MPI_UNAVAILABLE
 #endif
@@ -270,7 +377,6 @@
 #endif
 
 
-OMPI_DECLSPEC extern union dt_elem_desc ompi_datatype_predefined_elem_desc[2 * OMPI_DATATYPE_MPI_MAX_PREDEFINED];
 extern const ompi_datatype_t* ompi_datatype_basicDatatypes[OMPI_DATATYPE_MPI_MAX_PREDEFINED];
 
 /* There 3 types of predefined data types.
@@ -287,17 +393,17 @@ extern const ompi_datatype_t* ompi_datatype_basicDatatypes[OMPI_DATATYPE_MPI_MAX
  */
 
 #define OMPI_DATATYPE_EMPTY_DATA(NAME)                                               \
-    OMPI_DATATYPE_MPI_ ## NAME /*id*/,                                               \
-    0 /*d_f_to_c_index*/,                                                            \
-    NULL /*d_keyhash*/,                                                              \
-    NULL /*args*/,                                                                   \
-    NULL /*packed_description*/,                                                     \
-    "MPI_" # NAME /*name*/
+    .id = OMPI_DATATYPE_MPI_ ## NAME,                                                \
+    .d_f_to_c_index = 0,                                                             \
+    .d_keyhash = NULL,                                                               \
+    .args = NULL,                                                                    \
+    .packed_description = NULL,                                                      \
+    .name = "MPI_" # NAME
 
 #define OMPI_DATATYPE_INITIALIZER_UNAVAILABLE(FLAGS)                                 \
     OPAL_DATATYPE_INITIALIZER_UNAVAILABLE(FLAGS)
 
-#define OMPI_DATATYPE_INIT_PREDEFINED_BASIC_TYPE( TYPE, NAME, FLAGS )                \
+#define OMPI_DATATYPE_INIT_PREDEFINED_BASIC_TYPE_X( TYPE, NAME, FLAGS )              \
     { /*ompi_predefined_datatype_t*/                                                 \
         { /* ompi_datatype_t */                                                      \
             OMPI_DATATYPE_INITIALIZER_ ## TYPE (OMPI_DATATYPE_FLAG_PREDEFINED |      \
@@ -306,6 +412,10 @@ extern const ompi_datatype_t* ompi_datatype_basicDatatypes[OMPI_DATATYPE_MPI_MAX
         },                                                                           \
         {0, } /* padding */                                                          \
     }
+
+/* Preprocessor hack to make sure TYPE gets expanded properly */
+#define OMPI_DATATYPE_INIT_PREDEFINED_BASIC_TYPE(TYPE, NAME, FLAGS) OMPI_DATATYPE_INIT_PREDEFINED_BASIC_TYPE_X(TYPE, NAME, FLAGS)
+
 /*
  * Two macros for convenience
  */
@@ -326,50 +436,36 @@ extern const ompi_datatype_t* ompi_datatype_basicDatatypes[OMPI_DATATYPE_MPI_MAX
     OMPI_DATATYPE_INIT_UNAVAILABLE(NAME, FLAGS)
 
 
-#if OMPI_WANT_F77_BINDINGS
-/*
- * For Fortran, we need to pass information, such as ALIGNMENT and SIZE as well
- * Therefore, for initialization at compile-time, pass this data as well.
- *
- * However, there is no underlying OPAL-TYPE, therefore we just pass NAME, SIZE,
- * ALIGN and the FLAGS. Additionally, ONLY for Fortran we need the
- * ompi_datatype_predefined_elem_desc for the additional types.
- */
-#define OMPI_DATATYPE_INIT_DESC_PREDEFINED(TYPE, SIZE)                               \
-    {                                                                                \
-        1 /*length*/, 1 /*used*/,                                                    \
-        &(ompi_datatype_predefined_elem_desc[2 * OPAL_DATATYPE_ ## TYPE ## SIZE]) /*desc*/ \
-    }
-
+#if OMPI_BUILD_FORTRAN_BINDINGS
 /*
  * Fortran types are based on the underlying OPAL types: They share the ID -- however,
  * the alignment is overwritten.
  */
 #define OMPI_DATATYPE_INITIALIZER_FORTRAN( TYPE, NAME, SIZE, ALIGN, FLAGS )          \
     {                                                                                \
-        OPAL_OBJ_STATIC_INIT(opal_datatype_t),                                       \
-        OPAL_DATATYPE_FLAG_BASIC |                                                   \
+        .super = OPAL_OBJ_STATIC_INIT(opal_datatype_t),                              \
+        .flags = OPAL_DATATYPE_FLAG_BASIC |                                          \
             OMPI_DATATYPE_FLAG_PREDEFINED |                                          \
-            OMPI_DATATYPE_FLAG_DATA_FORTRAN | (FLAGS) /*flag*/,                      \
-        OPAL_DATATYPE_ ## TYPE ## SIZE /*id*/,                                       \
-        (((uint32_t)1)<<(OPAL_DATATYPE_ ## TYPE ## SIZE)) /*bdt_used*/,              \
-        SIZE /*size*/,                                                               \
-        0 /*true_lb*/, SIZE /*true_ub*/, 0 /*lb*/, SIZE /*ub*/,                      \
-        (ALIGN) /*align*/,                                                           \
-        1 /*nbElems*/,                                                               \
-        OPAL_DATATYPE_INIT_NAME(TYPE ## SIZE) /*name*/,                              \
-        OMPI_DATATYPE_INIT_DESC_PREDEFINED(TYPE, SIZE) /*desc*/,                     \
-        OMPI_DATATYPE_INIT_DESC_PREDEFINED(TYPE, SIZE) /*opt_desc*/,                 \
-        OPAL_DATATYPE_INIT_BTYPES_ARRAY_ ## TYPE ## SIZE /*btypes*/                  \
+            OMPI_DATATYPE_FLAG_DATA_FORTRAN | (FLAGS),                               \
+        .id = OPAL_DATATYPE_ ## TYPE ## SIZE,                                        \
+        .bdt_used = (((uint32_t)1)<<(OPAL_DATATYPE_ ## TYPE ## SIZE)),               \
+        .size = SIZE,                                                                \
+        .true_lb = 0, .true_ub = SIZE, .lb = 0, .ub = SIZE,                          \
+        .align = (ALIGN),                                                            \
+        .nbElems = 1,                                                                \
+        .name = OPAL_DATATYPE_INIT_NAME(TYPE ## SIZE),                               \
+        .desc = OPAL_DATATYPE_INIT_DESC_PREDEFINED(TYPE ## SIZE),                    \
+        .opt_desc = OPAL_DATATYPE_INIT_DESC_PREDEFINED(TYPE ## SIZE),                \
+	.btypes = OPAL_DATATYPE_INIT_BTYPES_ARRAY(TYPE ## SIZE)                      \
     }
 
 #define OMPI_DATATYPE_INIT_PREDEFINED_BASIC_TYPE_FORTRAN( TYPE, NAME, SIZE, ALIGN, FLAGS ) \
     { /*ompi_predefined_datatype_t*/                                                 \
-        { /*ompi_datatype_t*/                                                        \
-            OMPI_DATATYPE_INITIALIZER_FORTRAN( TYPE, NAME, SIZE, ALIGN, FLAGS) /*super*/, \
+        .dt = {                                                                      \
+            .super = OMPI_DATATYPE_INITIALIZER_FORTRAN( TYPE, NAME, SIZE, ALIGN, FLAGS), \
             OMPI_DATATYPE_EMPTY_DATA(NAME) /*id,d_f_to_c_index,d_keyhash,args,packed_description,name*/ \
         },                                                                           \
-        {0, } /* padding */                                                          \
+        .padding = {0, }                                                             \
     }
 #else
 #define OMPI_DATATYPE_INIT_PREDEFINED_BASIC_TYPE_FORTRAN( TYPE, NAME, SIZE, ALIGN, FLAGS ) \
@@ -383,10 +479,37 @@ extern const ompi_datatype_t* ompi_datatype_basicDatatypes[OMPI_DATATYPE_MPI_MAX
 
 #define OMPI_DATATYPE_INITIALIZER_LB                  OPAL_DATATYPE_INITIALIZER_LB
 #define OMPI_DATATYPE_INITIALIZER_UB                  OPAL_DATATYPE_INITIALIZER_UB
+
+#define OMPI_DATATYPE_INITIALIZER_INT8_T              OPAL_DATATYPE_INITIALIZER_INT1
+#define OMPI_DATATYPE_INITIALIZER_UINT8_T             OPAL_DATATYPE_INITIALIZER_UINT1
+#define OMPI_DATATYPE_INITIALIZER_INT16_T             OPAL_DATATYPE_INITIALIZER_INT2
+#define OMPI_DATATYPE_INITIALIZER_UINT16_T            OPAL_DATATYPE_INITIALIZER_UINT2
+#define OMPI_DATATYPE_INITIALIZER_INT32_T             OPAL_DATATYPE_INITIALIZER_INT4
+#define OMPI_DATATYPE_INITIALIZER_UINT32_T            OPAL_DATATYPE_INITIALIZER_UINT4
+#define OMPI_DATATYPE_INITIALIZER_INT64_T             OPAL_DATATYPE_INITIALIZER_INT8
+#define OMPI_DATATYPE_INITIALIZER_UINT64_T            OPAL_DATATYPE_INITIALIZER_UINT8
+
+#if SIZEOF_CHAR == 1
 #define OMPI_DATATYPE_INITIALIZER_CHAR                OPAL_DATATYPE_INITIALIZER_INT1
 #define OMPI_DATATYPE_INITIALIZER_UNSIGNED_CHAR       OPAL_DATATYPE_INITIALIZER_UINT1
 #define OMPI_DATATYPE_INITIALIZER_SIGNED_CHAR         OPAL_DATATYPE_INITIALIZER_INT1
 #define OMPI_DATATYPE_INITIALIZER_BYTE                OPAL_DATATYPE_INITIALIZER_UINT1
+#elif SIZEOF_CHAR == 2
+#define OMPI_DATATYPE_INITIALIZER_CHAR                OPAL_DATATYPE_INITIALIZER_INT2
+#define OMPI_DATATYPE_INITIALIZER_UNSIGNED_CHAR       OPAL_DATATYPE_INITIALIZER_UINT2
+#define OMPI_DATATYPE_INITIALIZER_SIGNED_CHAR         OPAL_DATATYPE_INITIALIZER_INT2
+#define OMPI_DATATYPE_INITIALIZER_BYTE                OPAL_DATATYPE_INITIALIZER_UINT2
+#elif SIZEOF_CHAR == 4
+#define OMPI_DATATYPE_INITIALIZER_CHAR                OPAL_DATATYPE_INITIALIZER_INT4
+#define OMPI_DATATYPE_INITIALIZER_UNSIGNED_CHAR       OPAL_DATATYPE_INITIALIZER_UINT4
+#define OMPI_DATATYPE_INITIALIZER_SIGNED_CHAR         OPAL_DATATYPE_INITIALIZER_INT4
+#define OMPI_DATATYPE_INITIALIZER_BYTE                OPAL_DATATYPE_INITIALIZER_UINT4
+#elif SIZEOF_CHAR == 8
+#define OMPI_DATATYPE_INITIALIZER_CHAR                OPAL_DATATYPE_INITIALIZER_INT8
+#define OMPI_DATATYPE_INITIALIZER_UNSIGNED_CHAR       OPAL_DATATYPE_INITIALIZER_UINT8
+#define OMPI_DATATYPE_INITIALIZER_SIGNED_CHAR         OPAL_DATATYPE_INITIALIZER_INT8
+#define OMPI_DATATYPE_INITIALIZER_BYTE                OPAL_DATATYPE_INITIALIZER_UINT8
+#endif
 
 #if SIZEOF_SHORT == 2
 #define OMPI_DATATYPE_INITIALIZER_SHORT               OPAL_DATATYPE_INITIALIZER_INT2
@@ -399,16 +522,6 @@ extern const ompi_datatype_t* ompi_datatype_basicDatatypes[OMPI_DATATYPE_MPI_MAX
 #define OMPI_DATATYPE_INITIALIZER_UNSIGNED_SHORT      OPAL_DATATYPE_INITIALIZER_UINT8
 #endif
 
-/*
- * Unfortunately, the following does not work:
- *
-#define OMPI_DATATYPE_INITIALIZER_INT                 OPAL_DATATYPE_INITIALIZER_INT ## SIZEOF_INT
-#define OMPI_DATATYPE_INITIALIZER_UINT                OPAL_DATATYPE_INITIALIZER_UINT ## SIZEOF_INT
-#define OMPI_DATATYPE_INT                             OPAL_DATATYPE_INT ## SIZEOF_INT
-#define OMPI_DATATYPE_UNSIGNED_INT                    OPAL_DATATYPE_UINT ## SIZEOF_INT
- * therefore do it the long way
- */
-
 #if SIZEOF_INT == 2
 #define OMPI_DATATYPE_INITIALIZER_INT                 OPAL_DATATYPE_INITIALIZER_INT2
 #define OMPI_DATATYPE_INITIALIZER_UNSIGNED_INT        OPAL_DATATYPE_INITIALIZER_UINT2
@@ -494,18 +607,15 @@ extern const ompi_datatype_t* ompi_datatype_basicDatatypes[OMPI_DATATYPE_MPI_MAX
 
 #define OMPI_DATATYPE_INITIALIZER_WCHAR               OPAL_DATATYPE_INITIALIZER_WCHAR
 
+#define OMPI_DATATYPE_INITIALIZER_C_FLOAT_COMPLEX       OPAL_DATATYPE_INITIALIZER_FLOAT_COMPLEX
+#define OMPI_DATATYPE_INITIALIZER_C_DOUBLE_COMPLEX      OPAL_DATATYPE_INITIALIZER_DOUBLE_COMPLEX
+#define OMPI_DATATYPE_INITIALIZER_C_LONG_DOUBLE_COMPLEX OPAL_DATATYPE_INITIALIZER_LONG_DOUBLE_COMPLEX
+
 /*
  * Following are the structured types, that cannot be represented
  * by one single OPAL basic type
  */
 #define OMPI_DATATYPE_FIRST_TYPE                      OPAL_DATATYPE_MAX_PREDEFINED
-#define OMPI_DATATYPE_COMPLEX                         (OMPI_DATATYPE_FIRST_TYPE+0)   /* Equal to OMPI_DATATYPE_FIRST_TYPE */
-#define OMPI_DATATYPE_DOUBLE_COMPLEX                  (OMPI_DATATYPE_FIRST_TYPE+1)
-#define OMPI_DATATYPE_LONG_DOUBLE_COMPLEX             (OMPI_DATATYPE_FIRST_TYPE+2)
-
-#define OMPI_DATATYPE_COMPLEX8                        (OMPI_DATATYPE_FIRST_TYPE+3)
-#define OMPI_DATATYPE_COMPLEX16                       (OMPI_DATATYPE_FIRST_TYPE+4)
-#define OMPI_DATATYPE_COMPLEX32                       (OMPI_DATATYPE_FIRST_TYPE+5)
 
 /*
  * Derived datatypes supposely contiguous
