@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2007-2009 Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2007-2013 Cisco Systems, Inc.  All rights reserved.
  * Copyright (c) 2007-2008 Chelsio, Inc. All rights reserved.
  * Copyright (c) 2008      Mellanox Technologies. All rights reserved.
  * Copyright (c) 2009      Sandia National Laboratories. All rights reserved.
@@ -228,7 +228,7 @@ static void rdmacm_contents_destructor(rdmacm_contents_t *contents)
  */
 static void rdmacm_component_register(void)
 {
-    rdmacm_priority = 30;
+    /* the priority is initialized in the declaration above */
     (void) mca_base_component_var_register(&mca_btl_openib_component.super.btl_version,
                                            "connect_rdmacm_priority",
                                            "The selection method priority for rdma_cm",
@@ -908,13 +908,13 @@ static int handle_connect_request(struct rdma_cm_event *event)
             goto out1;
         }
         OPAL_OUTPUT((-1, "Posted CTS receiver buffer (%p) for peer %s, qp index %d (QP num %d), WR ID %p, SG addr %p, len %d, lkey %d",
-                     (void*) wr->sg_list[0].addr,
+                     (void*)((uintptr_t*) wr->sg_list[0].addr),
                      (NULL == endpoint->endpoint_proc->proc_ompi->proc_hostname) ?
                      "unknown" : endpoint->endpoint_proc->proc_ompi->proc_hostname,
                      qpnum,
                      endpoint->qps[qpnum].qp->lcl_qp->qp_num,
-                     (void*) wr->wr_id,
-                     (void*) wr->sg_list[0].addr,
+                     (void*)((uintptr_t*) wr->wr_id),
+                     (void*)((uintptr_t*) wr->sg_list[0].addr),
                      wr->sg_list[0].length,
                      wr->sg_list[0].lkey));
     }
@@ -1068,10 +1068,8 @@ static int rdmacm_endpoint_finalize(struct mca_btl_base_endpoint_t *endpoint)
                 OPAL_OUTPUT((-1, "MAIN Main thread calling disconnect on ID %p",
                              (void*) ((id_context_t*) item2)->id));
                 ++num_to_wait_for;
-#ifndef __WINDOWS__
                 ompi_btl_openib_fd_run_in_service(call_disconnect_callback,
                                                   item2);
-#endif
             }
 	    /* remove_item returns the item before the item removed,
 	       meaning that the for list is still safe */
@@ -1087,10 +1085,8 @@ static int rdmacm_endpoint_finalize(struct mca_btl_base_endpoint_t *endpoint)
 
     /* Now wait for all the disconnect callbacks to occur */
     while (num_to_wait_for != disconnect_callbacks) {
-#ifndef __WINDOWS__
         ompi_btl_openib_fd_main_thread_drain();
         sched_yield();
-#endif
     }
 
     OPAL_OUTPUT((-1, "MAIN Endpoint finished finalizing"));
@@ -1351,8 +1347,8 @@ static int finish_connect(id_context_t *context)
     struct rdma_conn_param conn_param;
     private_data_t msg;
     int rc;
-    struct sockaddr *peeraddr, *localaddr;
-    uint32_t localipaddr, remoteipaddr;
+    struct sockaddr *peeraddr;
+    uint32_t remoteipaddr;
     uint16_t remoteport;
     modex_message_t *message;
 
@@ -1360,9 +1356,6 @@ static int finish_connect(id_context_t *context)
     peeraddr = rdma_get_peer_addr(context->id);
     remoteipaddr = ((struct sockaddr_in *)peeraddr)->sin_addr.s_addr;
 
-    localaddr = rdma_get_local_addr(context->id);
-    localipaddr = ((struct sockaddr_in *)localaddr)->sin_addr.s_addr;
-
     message = (modex_message_t *)
         context->endpoint->endpoint_remote_cpc_data->cbm_modex_message;
 
@@ -1392,7 +1385,7 @@ static int finish_connect(id_context_t *context)
                 goto out1;
             }
             OPAL_OUTPUT((-1, "Posted initiator CTS buffer (%p, length %d) for peer %s, qp index %d (QP num %d)",
-                         (void*) wr->sg_list[0].addr,
+                         (void*)((uintptr_t*) wr->sg_list[0].addr),
                          wr->sg_list[0].length,
                          (NULL == contents->endpoint->endpoint_proc->proc_ompi->proc_hostname) ?
                          "unknown" : contents->endpoint->endpoint_proc->proc_ompi->proc_hostname,
@@ -1521,7 +1514,7 @@ static int event_handler(struct rdma_cm_event *event)
     rdmacm_contents_t *contents;
     struct sockaddr *peeraddr, *localaddr;
     uint32_t peeripaddr, localipaddr;
-    int rc = -1, qpnum;
+    int rc = -1;
     ompi_btl_openib_ini_values_t ini;
     bool found;
 
@@ -1530,7 +1523,6 @@ static int event_handler(struct rdma_cm_event *event)
     }
 
     contents = context->contents;
-    qpnum = context->qpnum;
     localaddr = rdma_get_local_addr(event->id);
     peeraddr = rdma_get_peer_addr(event->id);
     localipaddr = ((struct sockaddr_in *)localaddr)->sin_addr.s_addr;
@@ -1995,10 +1987,8 @@ static int rdmacm_component_finalize(void)
     }
 
     if (NULL != event_channel) {
-#ifndef __WINDOWS__
         rc = ompi_btl_openib_fd_unmonitor(event_channel->fd,
                                           rdmacm_unmonitor, (void*) &barrier);
-#endif
         if (OMPI_SUCCESS != rc) {
             BTL_ERROR(("Error disabling fd monitor"));
         }
@@ -2006,9 +1996,7 @@ static int rdmacm_component_finalize(void)
         /* Wait for the service thread to stop monitoring the fd */
         OPAL_OUTPUT((-1, "MAIN rdmacm_component_finalize: waiting for thread to finish"));
         while (0 == barrier) {
-#ifndef __WINDOWS__
             sched_yield();
-#endif
         }
         OPAL_OUTPUT((-1, "MAIN rdmacm_component_finalize: thread finished"));
     }
@@ -2068,11 +2056,9 @@ static int rdmacm_component_init(void)
         return OMPI_ERR_UNREACH;
     }
 
-#ifndef __WINDOWS__
     /* Start monitoring the fd associated with the cm_device */
     ompi_btl_openib_fd_monitor(event_channel->fd, OPAL_EV_READ,
                                rdmacm_event_dispatch, NULL);
-#endif
 
     rdmacm_component_initialized = true;
     return OMPI_SUCCESS;
