@@ -5,6 +5,9 @@
  * Copyright (c) 2009      IBM Corporation.  All rights reserved.
  * Copyright (c) 2011-2014 Los Alamos National Security, LLC.  All rights
  *                         reserved.
+ * Copyright (c) 2014-2015 Research Organization for Information Science
+ *                         and Technology (RIST). All rights reserved.
+ * Copyright (c) 2014      Bull SAS.  All rights reserved.
  *
  * $COPYRIGHT$
  * 
@@ -69,6 +72,7 @@
 #include "connect/connect.h"
 
 #include "ompi/mca/mpool/grdma/mpool_grdma.h"
+#include "opal/util/sys_limits.h"
 
 #if (ENABLE_DYNAMIC_SL)
 #include "connect/btl_openib_connect_sl.h"
@@ -166,7 +170,7 @@ typedef struct {
 
     struct ibv_ah *ah;
 
-    bool           sent_req, recv_req, recv_resp;
+    bool           sent_req, recv_req, recv_resp, recv_comp;
 
     /* Has this endpoint's data been initialized */
     bool           udep_initialized, udep_created_qps;
@@ -315,6 +319,7 @@ static int udcm_send_request (mca_btl_base_endpoint_t *lcl_ep,
                               mca_btl_base_endpoint_t *rem_ep);
 
 static void udcm_send_timeout (evutil_socket_t fd, short event, void *arg);
+static int udcm_finish_connection (mca_btl_openib_endpoint_t *lcl_ep);
 
 /* XRC support */
 #if HAVE_XRC
@@ -323,7 +328,11 @@ static int udcm_xrc_start_connect (ompi_btl_openib_connect_base_module_t *cpc,
 static int udcm_xrc_restart_connect (mca_btl_base_endpoint_t *lcl_ep);
 static int udcm_xrc_send_qp_connect (mca_btl_openib_endpoint_t *lcl_ep, udcm_msg_hdr_t *msg_hdr);
 static int udcm_xrc_send_qp_create (mca_btl_base_endpoint_t *lcl_ep);
+#if OMPI_HAVE_CONNECTX_XRC_DOMAINS
+static int udcm_xrc_recv_qp_connect (mca_btl_openib_endpoint_t *lcl_ep, uint32_t qp_num);
+#else
 static int udcm_xrc_recv_qp_connect (mca_btl_openib_endpoint_t *lcl_ep);
+#endif
 static int udcm_xrc_recv_qp_create (mca_btl_openib_endpoint_t *lcl_ep, udcm_msg_hdr_t *msg_hdr);
 static int udcm_xrc_send_request (mca_btl_base_endpoint_t *lcl_ep, mca_btl_base_endpoint_t *rem_ep,
                                   uint8_t msg_type);
@@ -541,6 +550,13 @@ static int udcm_module_init (udcm_module_t *m, mca_btl_openib_module_t *btl)
     BTL_VERBOSE(("created cpc module %p for btl %p",
                  (void*)m, (void*)btl));
 
+    OBJ_CONSTRUCT(&m->cm_lock, opal_mutex_t);
+    OBJ_CONSTRUCT(&m->cm_send_lock, opal_mutex_t);
+    OBJ_CONSTRUCT(&m->cm_recv_msg_queue, opal_list_t);
+    OBJ_CONSTRUCT(&m->cm_recv_msg_queue_lock, opal_mutex_t);
+    OBJ_CONSTRUCT(&m->flying_messages, opal_list_t);
+    OBJ_CONSTRUCT(&m->cm_timeout_lock, opal_mutex_t);
+
     m->btl = btl;
 
     /* Create completion channel */
@@ -613,15 +629,6 @@ static int udcm_module_init (udcm_module_t *m, mca_btl_openib_module_t *btl)
                                udcm_cq_event_dispatch, m);
     m->channel_monitored = true;
 
-    OBJ_CONSTRUCT(&m->cm_lock, opal_mutex_t);
-    OBJ_CONSTRUCT(&m->cm_send_lock, opal_mutex_t);
-    OBJ_CONSTRUCT(&m->cm_recv_msg_queue, opal_list_t);
-    OBJ_CONSTRUCT(&m->cm_recv_msg_queue_lock, opal_mutex_t);
-
-    OBJ_CONSTRUCT(&m->flying_messages, opal_list_t);
-
-    OBJ_CONSTRUCT(&m->cm_timeout_lock, opal_mutex_t);
-
     udcm_timeout_tv.tv_sec  = udcm_timeout / 1000000;
     udcm_timeout_tv.tv_usec = udcm_timeout - 1000000 *
         udcm_timeout_tv.tv_sec;
@@ -932,7 +939,7 @@ static int udcm_module_allocate_buffers (udcm_module_t *m)
                                           UDCM_GRH_SIZE);
 
     m->cm_buffer = NULL;
-    posix_memalign ((void **)&m->cm_buffer, sysconf(_SC_PAGESIZE),
+    posix_memalign ((void **)&m->cm_buffer, (size_t)opal_getpagesize(),
                     total_size);
     if (NULL == m->cm_buffer) {
         BTL_ERROR(("malloc failed! errno = %d", errno));
@@ -1661,9 +1668,16 @@ static int udcm_handle_ack (udcm_module_t *m, const uintptr_t ctx, const uint16_
         BTL_VERBOSE(("found matching message"));
         found = true;
 
-        /* found it */
-        opal_list_remove_item (&m->flying_messages, &msg->super);
-        OBJ_RELEASE(msg);
+        /* mark that this event is not active anymore */
+        msg->event_active = false;
+
+        /* there is a possibility this event is being handled by another thread right now. it
+         * should be safe to activate the event even in this case. the callback will handle
+         * releasing the message. this is done to avoid a race between the message handling
+         * thread and the thread progressing libevent. if the message handler is ever put
+         * in the event base then it will be safe to just release the message here but that
+         * is not the case atm. */
+        opal_event_active (&msg->event, 0, 0);
 
         break;
     }
@@ -1731,6 +1745,10 @@ static int udcm_handle_connect(mca_btl_openib_endpoint_t *lcl_ep,
             break;
         }
 
+        if (udep->recv_comp) {
+            udcm_finish_connection (lcl_ep);
+        }
+
         opal_mutex_unlock (&udep->udep_lock);
 
         return OMPI_SUCCESS;
@@ -1788,6 +1806,20 @@ static int udcm_finish_connection (mca_btl_openib_endpoint_t *lcl_ep)
     return OMPI_SUCCESS;
 }
 
+static int udcm_handle_complete (mca_btl_openib_endpoint_t *lcl_ep)
+{
+    udcm_endpoint_t *udep = UDCM_ENDPOINT_DATA(lcl_ep);
+
+    udep->recv_comp = true;
+    if (udep->recv_req) {
+        udcm_finish_connection (lcl_ep);
+    } else {
+        OPAL_THREAD_UNLOCK(&lcl_ep->endpoint_lock);
+    }
+
+    return OMPI_SUCCESS;
+}
+
 /* mark: message processing */
 
 static int udcm_process_messages (struct ibv_cq *event_cq, udcm_module_t *m)
@@ -1874,6 +1906,7 @@ static int udcm_process_messages (struct ibv_cq *event_cq, udcm_module_t *m)
         if (OMPI_SUCCESS != udcm_endpoint_init_data (lcl_ep)) {
             BTL_ERROR(("could not initialize cpc data for endpoint"));
             udcm_module_post_one_recv (m, msg_num);
+            opal_mutex_unlock (&udep->udep_lock);
             continue;
         }
 
@@ -1931,7 +1964,10 @@ static int udcm_process_messages (struct ibv_cq *event_cq, udcm_module_t *m)
 
             if (UDCM_MESSAGE_XCONNECT2 == message->hdr.type) {
                 /* save the qp number for unregister */
+#if ! OMPI_HAVE_CONNECTX_XRC_DOMAINS
                 lcl_ep->xrc_recv_qp_num = message->hdr.data.xreq.rem_qp_num;
+#endif
+
             }
         }
 #endif
@@ -2032,7 +2068,7 @@ static void *udcm_message_callback (void *context)
             OPAL_THREAD_UNLOCK(&lcl_ep->endpoint_lock);
             break;
         case UDCM_MESSAGE_COMPLETE:
-            udcm_finish_connection (lcl_ep);
+            udcm_handle_complete (lcl_ep);
             break;
 #if HAVE_XRC
         case UDCM_MESSAGE_XRESPONSE2:
@@ -2096,16 +2132,16 @@ static void udcm_send_timeout (evutil_socket_t fd, short event, void *arg)
     udcm_module_t *m = UDCM_ENDPOINT_MODULE(lcl_ep);
 
     opal_mutex_lock (&m->cm_timeout_lock);
+    opal_list_remove_item (&m->flying_messages, &msg->super);
+    opal_mutex_unlock (&m->cm_timeout_lock);
 
     if (m->cm_exiting || !msg->event_active) {
-        /* lost a race with the thread handling the acks or exiting */
-        opal_mutex_unlock (&m->cm_timeout_lock);
+        /* we are exiting or the event is no longer valid */
+        OBJ_RELEASE(msg);
         return;
     }
 
-    opal_list_remove_item (&m->flying_messages, &msg->super);
     msg->event_active = false;
-    opal_mutex_unlock (&m->cm_timeout_lock);
 
     do {
         BTL_VERBOSE(("send for message to 0x%04x:0x%08x timed out",
@@ -2372,7 +2408,11 @@ static int udcm_xrc_send_qp_create (mca_btl_base_endpoint_t *lcl_ep)
     uint32_t send_wr;
     struct ibv_qp **qp;
     uint32_t *psn;
+#if OMPI_HAVE_CONNECTX_XRC_DOMAINS
+    struct ibv_qp_init_attr_ex qp_init_attr;
+#else
     struct ibv_qp_init_attr qp_init_attr;
+#endif
     struct ibv_qp_attr attr;
     int ret;
     size_t req_inline;
@@ -2389,7 +2429,11 @@ static int udcm_xrc_send_qp_create (mca_btl_base_endpoint_t *lcl_ep)
     send_wr = lcl_ep->ib_addr->qp->sd_wqe +
         (mca_btl_openib_component.use_eager_rdma ?
          mca_btl_openib_component.max_eager_rdma : 0);
+#if OMPI_HAVE_CONNECTX_XRC_DOMAINS
+    memset(&qp_init_attr, 0, sizeof(struct ibv_qp_init_attr_ex));
+#else
     memset(&qp_init_attr, 0, sizeof(struct ibv_qp_init_attr));
+#endif
     memset(&attr, 0, sizeof(struct ibv_qp_attr));
 
     qp_init_attr.send_cq = qp_init_attr.recv_cq = openib_btl->device->ib_cq[prio];
@@ -2402,9 +2446,16 @@ static int udcm_xrc_send_qp_create (mca_btl_base_endpoint_t *lcl_ep)
     qp_init_attr.cap.max_send_sge = 1;
     /* this one is ignored by driver */
     qp_init_attr.cap.max_recv_sge = 1; /* we do not use SG list */
+#if OMPI_HAVE_CONNECTX_XRC_DOMAINS
+    qp_init_attr.qp_type = IBV_QPT_XRC_SEND;
+    qp_init_attr.comp_mask = IBV_QP_INIT_ATTR_PD;
+    qp_init_attr.pd = openib_btl->device->ib_pd;
+    *qp = ibv_create_qp_ex(openib_btl->device->ib_dev_context, &qp_init_attr);
+#else
     qp_init_attr.qp_type = IBV_QPT_XRC;
     qp_init_attr.xrc_domain = openib_btl->device->xrc_domain;
     *qp = ibv_create_qp(openib_btl->device->ib_pd, &qp_init_attr);
+#endif
     if (NULL == *qp) {
         opal_show_help("help-mpi-btl-openib-cpc-base.txt",
                        "ibv_create_qp failed", true,
@@ -2451,11 +2502,34 @@ static int udcm_xrc_send_qp_create (mca_btl_base_endpoint_t *lcl_ep)
 /* mark: xrc receive qp */
 
 /* Recv qp connect */
+#if OMPI_HAVE_CONNECTX_XRC_DOMAINS
+static int udcm_xrc_recv_qp_connect (mca_btl_openib_endpoint_t *lcl_ep, uint32_t qp_num)
+#else
 static int udcm_xrc_recv_qp_connect (mca_btl_openib_endpoint_t *lcl_ep)
+#endif
 {
     mca_btl_openib_module_t *openib_btl = lcl_ep->endpoint_btl;
-    int ret;
 
+#if OMPI_HAVE_CONNECTX_XRC_DOMAINS
+    struct ibv_qp_open_attr attr;
+    memset(&attr, 0, sizeof(struct ibv_qp_open_attr));
+    attr.comp_mask = IBV_QP_OPEN_ATTR_NUM | IBV_QP_OPEN_ATTR_XRCD | IBV_QP_OPEN_ATTR_TYPE;
+    attr.qp_num = qp_num;
+    attr.qp_type = IBV_QPT_XRC_RECV;
+    attr.xrcd = openib_btl->device->xrcd;
+    BTL_VERBOSE(("Connecting Recv QP\n"));
+    lcl_ep->xrc_recv_qp = ibv_open_qp(openib_btl->device->ib_dev_context, &attr);
+    if (NULL == lcl_ep->xrc_recv_qp) { /* failed to regester the qp, so it is already die and we should create new one */
+       /* Return NOT READY !!!*/
+        BTL_ERROR(("Failed to register qp_num: %d , get error: %s (%d)\n. Replying with RNR",
+                   lcl_ep->xrc_recv_qp->qp_num, strerror(errno), errno));
+        return OMPI_ERROR;
+    } else {
+        BTL_VERBOSE(("Connected to XRC Recv qp [%d]", lcl_ep->xrc_recv_qp->qp_num));
+        return OMPI_SUCCESS;
+    }
+#else
+    int ret;
     BTL_VERBOSE(("Connecting receive qp: %d", lcl_ep->xrc_recv_qp_num));
     ret = ibv_reg_xrc_rcv_qp(openib_btl->device->xrc_domain, lcl_ep->xrc_recv_qp_num);
     if (ret) { /* failed to regester the qp, so it is already die and we should create new one */
@@ -2465,6 +2539,7 @@ static int udcm_xrc_recv_qp_connect (mca_btl_openib_endpoint_t *lcl_ep)
                    lcl_ep->xrc_recv_qp_num, strerror(ret), ret));
         return OMPI_ERROR;
     }
+#endif
 
     return OMPI_SUCCESS;
 }
@@ -2473,12 +2548,29 @@ static int udcm_xrc_recv_qp_connect (mca_btl_openib_endpoint_t *lcl_ep)
 static int udcm_xrc_recv_qp_create (mca_btl_openib_endpoint_t *lcl_ep, udcm_msg_hdr_t *msg_hdr)
 {
     mca_btl_openib_module_t* openib_btl = lcl_ep->endpoint_btl;
+#if OMPI_HAVE_CONNECTX_XRC_DOMAINS
+    struct ibv_qp_init_attr_ex qp_init_attr;
+#else
     struct ibv_qp_init_attr qp_init_attr;
+#endif
     struct ibv_qp_attr attr;
     int ret;
 
     BTL_VERBOSE(("creating xrc receive qp"));
 
+#if OMPI_HAVE_CONNECTX_XRC_DOMAINS
+    memset(&qp_init_attr, 0, sizeof(struct ibv_qp_init_attr_ex));
+    qp_init_attr.qp_type = IBV_QPT_XRC_RECV;
+    qp_init_attr.comp_mask = IBV_QP_INIT_ATTR_XRCD;
+    qp_init_attr.xrcd = openib_btl->device->xrcd;
+    lcl_ep->xrc_recv_qp = ibv_create_qp_ex(openib_btl->device->ib_dev_context,
+                                           &qp_init_attr);
+    if (NULL == lcl_ep->xrc_recv_qp) {
+        BTL_ERROR(("Error creating XRC recv QP, errno says: %s [%d]",
+                    strerror(errno), errno));
+        return OMPI_ERROR;
+    }
+#else
     memset(&qp_init_attr, 0, sizeof(struct ibv_qp_init_attr));
     /* Only xrc_domain is required, all other are ignored */
     qp_init_attr.xrc_domain = openib_btl->device->xrc_domain;
@@ -2488,12 +2580,26 @@ static int udcm_xrc_recv_qp_create (mca_btl_openib_endpoint_t *lcl_ep, udcm_msg_
                    lcl_ep->xrc_recv_qp_num, strerror(ret), ret));
         return OMPI_ERROR;
     }
+#endif
 
     memset(&attr, 0, sizeof(struct ibv_qp_attr));
     attr.qp_state = IBV_QPS_INIT;
     attr.pkey_index = openib_btl->pkey_index;
     attr.port_num = openib_btl->port_num;
     attr.qp_access_flags = IBV_ACCESS_REMOTE_WRITE | IBV_ACCESS_REMOTE_READ;
+#if OMPI_HAVE_CONNECTX_XRC_DOMAINS
+    ret = ibv_modify_qp(lcl_ep->xrc_recv_qp,
+            &attr,
+            IBV_QP_STATE|
+            IBV_QP_PKEY_INDEX|
+            IBV_QP_PORT|
+            IBV_QP_ACCESS_FLAGS);
+    if (ret) {
+        BTL_ERROR(("Error modifying XRC recv QP to IBV_QPS_INIT, errno says: %s [%d]",
+                    strerror(ret), ret));
+        return OMPI_ERROR;
+    }
+#else
     ret = ibv_modify_xrc_rcv_qp(openib_btl->device->xrc_domain,
                                 lcl_ep->xrc_recv_qp_num, &attr,
                                 IBV_QP_STATE | IBV_QP_PKEY_INDEX |
@@ -2504,6 +2610,7 @@ static int udcm_xrc_recv_qp_create (mca_btl_openib_endpoint_t *lcl_ep, udcm_msg_
         while(1);
         return OMPI_ERROR;
     }
+#endif
 
     memset(&attr, 0, sizeof(struct ibv_qp_attr));
     attr.qp_state           = IBV_QPS_RTR;
@@ -2524,7 +2631,11 @@ static int udcm_xrc_recv_qp_create (mca_btl_openib_endpoint_t *lcl_ep, udcm_msg_
     /* if user enabled dynamic SL, get it from PathRecord */
     if (0 != mca_btl_openib_component.ib_path_record_service_level) {
         int rc = btl_openib_connect_get_pathrecord_sl(
+#if OMPI_HAVE_CONNECTX_XRC_DOMAINS
+                                                      openib_btl->device->xrcd->context,
+#else
                                                       openib_btl->device->xrc_domain->context,
+#endif
                                                       attr.ah_attr.port_num,
                                                       openib_btl->lid,
                                                       attr.ah_attr.dlid);
@@ -2535,6 +2646,22 @@ static int udcm_xrc_recv_qp_create (mca_btl_openib_endpoint_t *lcl_ep, udcm_msg_
     }
 #endif
 
+#if OMPI_HAVE_CONNECTX_XRC_DOMAINS
+    ret = ibv_modify_qp(lcl_ep->xrc_recv_qp,
+            &attr,
+            IBV_QP_STATE|
+            IBV_QP_AV|
+            IBV_QP_PATH_MTU|
+            IBV_QP_DEST_QPN|
+            IBV_QP_RQ_PSN|
+            IBV_QP_MAX_DEST_RD_ATOMIC|
+            IBV_QP_MIN_RNR_TIMER);
+    if (ret) {
+        BTL_ERROR(("Error modifying XRC recv QP to IBV_QPS_RTR, errno says: %s [%d]",
+                    strerror(ret), ret));
+        return OMPI_ERROR;
+    }
+#else
     ret = ibv_modify_xrc_rcv_qp(openib_btl->device->xrc_domain,
                                 lcl_ep->xrc_recv_qp_num,
                                 &attr,
@@ -2550,9 +2677,14 @@ static int udcm_xrc_recv_qp_create (mca_btl_openib_endpoint_t *lcl_ep, udcm_msg_
                    lcl_ep->xrc_recv_qp_num, strerror(ret), ret));
         return OMPI_ERROR;
     }
+#endif
 #if OPAL_HAVE_THREADS
     if (APM_ENABLED) {
+#if OMPI_HAVE_CONNECTX_XRC_DOMAINS
+        mca_btl_openib_load_apm(lcl_ep->xrc_recv_qp, lcl_ep);
+#else
         mca_btl_openib_load_apm_xrc_rcv(lcl_ep->xrc_recv_qp_num, lcl_ep);
+#endif
     }
 #endif
 
@@ -2617,14 +2749,29 @@ static int udcm_xrc_send_xresponse (mca_btl_base_endpoint_t *lcl_ep, mca_btl_bas
     msg->data->hdr.data.xres.rem_ep_index = htonl(lcl_ep->index);
 
     if (UDCM_MESSAGE_XRESPONSE == msg_type) {
+#if OMPI_HAVE_CONNECTX_XRC_DOMAINS
+        BTL_VERBOSE(("Sending qp: %d, psn: %d", lcl_ep->xrc_recv_qp->qp_num, lcl_ep->xrc_recv_psn));
+        msg->data->hdr.data.xres.rem_qp_num = htonl(lcl_ep->xrc_recv_qp->qp_num);
+        msg->data->hdr.data.xres.rem_psn    = htonl(lcl_ep->xrc_recv_psn);
+#else
         BTL_VERBOSE(("Sending qp: %d, psn: %d", lcl_ep->xrc_recv_qp_num, lcl_ep->xrc_recv_psn));
         msg->data->hdr.data.xres.rem_qp_num = htonl(lcl_ep->xrc_recv_qp_num);
         msg->data->hdr.data.xres.rem_psn    = htonl(lcl_ep->xrc_recv_psn);
+#endif
     }
 
     for (int i = 0; i < mca_btl_openib_component.num_xrc_qps; ++i) {
+#if OMPI_HAVE_CONNECTX_XRC_DOMAINS
+        uint32_t srq_num;
+        if (ibv_get_srq_num(lcl_ep->endpoint_btl->qps[i].u.srq_qp.srq, &srq_num)) {
+            BTL_ERROR(("BTL openib XOOB internal error: can't get srq num"));
+        }
+        BTL_VERBOSE(("Sending srq[%d] num  = %d", i, srq_num));
+        msg->data->qps[i].qp_num = htonl(srq_num);
+#else
         BTL_VERBOSE(("Sending srq[%d] num  = %d", i, lcl_ep->endpoint_btl->qps[i].u.srq_qp.srq->xrc_srq_num));
         msg->data->qps[i].qp_num = htonl(lcl_ep->endpoint_btl->qps[i].u.srq_qp.srq->xrc_srq_num);
+#endif
     }
 
     rc = udcm_post_send (lcl_ep, msg->data, m->msg_length, 0);
@@ -2664,7 +2811,11 @@ static int udcm_xrc_handle_xconnect (mca_btl_openib_endpoint_t *lcl_ep, udcm_msg
 
         if (UDCM_MESSAGE_XCONNECT2 == msg_hdr->type) {
             response_type = UDCM_MESSAGE_XRESPONSE2;
+#if OMPI_HAVE_CONNECTX_XRC_DOMAINS
+            rc = udcm_xrc_recv_qp_connect (lcl_ep, msg_hdr->data.xreq.rem_qp_num);
+#else
             rc = udcm_xrc_recv_qp_connect (lcl_ep);
+#endif
             if (OMPI_SUCCESS != rc) {
                 /* return not ready. remote side will retry */
                 rej_reason = UDCM_REJ_NOT_READY;
