@@ -11,9 +11,9 @@
  *                         All rights reserved.
  * Copyright (c) 2006      Sandia National Laboratories. All rights
  *                         reserved.
- * Copyright (c) 2008-2013 Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2008-2015 Cisco Systems, Inc.  All rights reserved.
  * Copyright (c) 2012      Los Alamos National Security, LLC.  All rights
- *                         reserved. 
+ *                         reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -21,16 +21,19 @@
  * $HEADER$
  */
 
-#include "ompi_config.h"
-
-#include <infiniband/verbs.h>
+#include "opal_config.h"
 
 #include "opal_stdint.h"
 
-#include "ompi/constants.h"
+#include "opal/constants.h"
+
+#if BTL_IN_OPAL
+#include "opal/mca/btl/btl.h"
+#include "opal/mca/btl/base/base.h"
+#else
 #include "ompi/mca/btl/btl.h"
 #include "ompi/mca/btl/base/base.h"
-#include "ompi/mca/common/verbs/common_verbs.h"
+#endif
 
 #include "btl_usnic.h"
 #include "btl_usnic_frag.h"
@@ -44,10 +47,10 @@
  * Return the WQE and also return the segment if no ACK pending
  */
 void
-ompi_btl_usnic_frag_send_complete(ompi_btl_usnic_module_t *module,
-                                    ompi_btl_usnic_send_segment_t *sseg)
+opal_btl_usnic_frag_send_complete(opal_btl_usnic_module_t *module,
+                                    opal_btl_usnic_send_segment_t *sseg)
 {
-    ompi_btl_usnic_send_frag_t *frag;
+    opal_btl_usnic_send_frag_t *frag;
 
     frag = sseg->ss_parent_frag;
 
@@ -56,14 +59,15 @@ ompi_btl_usnic_frag_send_complete(ompi_btl_usnic_module_t *module,
     --frag->sf_seg_post_cnt;
 
     /* checks for returnability made inside */
-    ompi_btl_usnic_send_frag_return_cond(module, frag);
+    opal_btl_usnic_send_frag_return_cond(module, frag);
 
     /* do bookkeeping */
     ++frag->sf_endpoint->endpoint_send_credits;
-    ++module->mod_channels[sseg->ss_channel].sd_wqe;
 
     /* see if this endpoint needs to be made ready-to-send */
-    ompi_btl_usnic_check_rts(frag->sf_endpoint);
+    opal_btl_usnic_check_rts(frag->sf_endpoint);
+
+    ++module->mod_channels[sseg->ss_channel].credits;
 }
 
 /*
@@ -71,10 +75,10 @@ ompi_btl_usnic_frag_send_complete(ompi_btl_usnic_module_t *module,
  * Return the WQE and also return the segment if no ACK pending
  */
 void
-ompi_btl_usnic_chunk_send_complete(ompi_btl_usnic_module_t *module,
-                                    ompi_btl_usnic_send_segment_t *sseg)
+opal_btl_usnic_chunk_send_complete(opal_btl_usnic_module_t *module,
+                                    opal_btl_usnic_send_segment_t *sseg)
 {
-    ompi_btl_usnic_send_frag_t *frag;
+    opal_btl_usnic_send_frag_t *frag;
 
     frag = sseg->ss_parent_frag;
 
@@ -83,19 +87,20 @@ ompi_btl_usnic_chunk_send_complete(ompi_btl_usnic_module_t *module,
     --frag->sf_seg_post_cnt;
 
     if (sseg->ss_send_posted == 0 && !sseg->ss_ack_pending) {
-        ompi_btl_usnic_release_send_segment(module, frag, sseg);
+        opal_btl_usnic_release_send_segment(module, frag, sseg);
     }
 
     /* done with whole fragment? */
     /* checks for returnability made inside */
-    ompi_btl_usnic_send_frag_return_cond(module, frag);
+    opal_btl_usnic_send_frag_return_cond(module, frag);
 
     /* do bookkeeping */
     ++frag->sf_endpoint->endpoint_send_credits;
-    ++module->mod_channels[sseg->ss_channel].sd_wqe;
 
     /* see if this endpoint needs to be made ready-to-send */
-    ompi_btl_usnic_check_rts(frag->sf_endpoint);
+    opal_btl_usnic_check_rts(frag->sf_endpoint);
+
+    ++module->mod_channels[sseg->ss_channel].credits;
 }
 
 /* Responsible for completing non-fastpath parts of a put or send operation,
@@ -105,59 +110,59 @@ ompi_btl_usnic_chunk_send_complete(ompi_btl_usnic_module_t *module,
  * This routine lives in this file to help prevent automatic inlining by the
  * compiler.
  *
- * The "tag" only applies to sends. */
+ * The "tag" only applies to sends.
+ */
 int
-ompi_btl_usnic_finish_put_or_send(
-    ompi_btl_usnic_module_t *module,
-    ompi_btl_usnic_endpoint_t *endpoint,
-    ompi_btl_usnic_send_frag_t *frag,
+opal_btl_usnic_finish_put_or_send(
+    opal_btl_usnic_module_t *module,
+    opal_btl_usnic_endpoint_t *endpoint,
+    opal_btl_usnic_send_frag_t *frag,
     mca_btl_base_tag_t tag)
 {
     int rc;
-    ompi_btl_usnic_small_send_frag_t *sfrag;
-    ompi_btl_usnic_send_segment_t *sseg;
+    opal_btl_usnic_small_send_frag_t *sfrag;
+    opal_btl_usnic_send_segment_t *sseg;
 
     /*
      * If this is small, need to do the copyin now.
      * We don't do this earlier in case we got lucky and were
      * able to do an inline send.  We did not, so here we are...
      */
-    if (frag->sf_base.uf_type == OMPI_BTL_USNIC_FRAG_SMALL_SEND) {
+    if (frag->sf_base.uf_type == OPAL_BTL_USNIC_FRAG_SMALL_SEND) {
 
-        sfrag = (ompi_btl_usnic_small_send_frag_t *)frag;
+        sfrag = (opal_btl_usnic_small_send_frag_t *)frag;
         sseg = &sfrag->ssf_segment;
 
         /* Copy in user data if there is any, collapsing 2 segments into 1.
          * We already packed via the convertor if necessary, so we only need to
          * handle the simple memcpy case here.
          */
-        if (frag->sf_base.uf_base.des_src_cnt > 1) {
+        if (frag->sf_base.uf_base.USNIC_SEND_LOCAL_COUNT > 1) {
             /* no convertor */
-            assert(NULL != frag->sf_base.uf_src_seg[1].seg_addr.pval);
+            assert(NULL != frag->sf_base.uf_local_seg[1].seg_addr.pval);
 
-            memcpy(((char *)(intptr_t)frag->sf_base.uf_src_seg[0].seg_addr.lval +
-                        frag->sf_base.uf_src_seg[0].seg_len),
-                    frag->sf_base.uf_src_seg[1].seg_addr.pval,
-                    frag->sf_base.uf_src_seg[1].seg_len);
+            memcpy(((char *)(intptr_t)frag->sf_base.uf_local_seg[0].seg_addr.lval +
+                        frag->sf_base.uf_local_seg[0].seg_len),
+                    frag->sf_base.uf_local_seg[1].seg_addr.pval,
+                    frag->sf_base.uf_local_seg[1].seg_len);
 
             /* update 1st segment length */
-            frag->sf_base.uf_base.des_src_cnt = 1;
-            frag->sf_base.uf_src_seg[0].seg_len +=
-                frag->sf_base.uf_src_seg[1].seg_len;
+            frag->sf_base.uf_base.USNIC_SEND_LOCAL_COUNT = 1;
+            frag->sf_base.uf_local_seg[0].seg_len +=
+                frag->sf_base.uf_local_seg[1].seg_len;
         }
 
-        sseg->ss_base.us_sg_entry[0].length =
-            sizeof(ompi_btl_usnic_btl_header_t) + frag->sf_size;
+        sseg->ss_len = sizeof(opal_btl_usnic_btl_header_t) + frag->sf_size;
 
         /* use standard channel */
         sseg->ss_channel = USNIC_DATA_CHANNEL;
         sseg->ss_base.us_btl_header->tag = tag;
     } else {
-        ompi_btl_usnic_large_send_frag_t *lfrag;
+        opal_btl_usnic_large_send_frag_t *lfrag;
 
         /* Save info about the frag so that future invocations of
          * usnic_handle_large_send can generate segments to put on the wire. */
-        lfrag = (ompi_btl_usnic_large_send_frag_t *)frag;
+        lfrag = (opal_btl_usnic_large_send_frag_t *)frag;
         lfrag->lsf_tag = tag;
         lfrag->lsf_cur_offset = 0;
         lfrag->lsf_cur_ptr = lfrag->lsf_des_src[0].seg_addr.pval;
@@ -174,6 +179,6 @@ ompi_btl_usnic_finish_put_or_send(
     }
 
     /* queue this fragment into the send engine */
-    rc = ompi_btl_usnic_endpoint_enqueue_frag(endpoint, frag);
+    rc = opal_btl_usnic_endpoint_enqueue_frag(endpoint, frag);
     return rc;
 }
