@@ -1,5 +1,7 @@
 /**
-  Copyright (c) 2011 Mellanox Technologies. All rights reserved.
+  Copyright (c) 2011      Mellanox Technologies. All rights reserved.
+  Copyright (c) 2015      Research Organization for Information Science
+                          and Technology (RIST). All rights reserved.
   $COPYRIGHT$
 
   Additional copyrights may follow
@@ -14,6 +16,7 @@
 
 #include "mpi.h"
 #include "opal/mca/mca.h"
+#include "opal/memoryhooks/memory.h"
 #include "ompi/mca/coll/coll.h"
 #include "ompi/request/request.h"
 #include "ompi/mca/pml/pml.h"
@@ -22,11 +25,14 @@
 #include "ompi/attribute/attribute.h"
 #include "ompi/op/op.h"
 
-#include "hcoll_api.h"
-#include "hcoll_constants.h"
+#include "hcoll/api/hcoll_api.h"
+#include "hcoll/api/hcoll_constants.h"
 
 
 #include "coll_hcoll_debug.h"
+#ifndef HCOLL_VERSION
+#define HCOLL_VERSION(major, minor) (((major)<<HCOLL_MAJOR_BIT)|((minor)<<HCOLL_MINOR_BIT))
+#endif
 BEGIN_C_DECLS
 
 
@@ -62,11 +68,19 @@ struct mca_coll_hcoll_component_t {
     /** Whether or not hcoll_init was ever called */
     bool libhcoll_initialized;
 
+    bool using_mem_hooks;
+
     /** MCA parameter: ON/OFF user defined datatype through HCOLL */
     int   hcoll_datatype_fallback;
 
+#if HCOLL_API >= HCOLL_VERSION(3,2)
+    /* hcoll init options */
+    hcoll_init_opts_t *init_opts;
+#endif
+
     /* FCA global stuff */
     mca_coll_hcoll_ops_t hcoll_ops;
+
     ompi_free_list_t requests;
 };
 typedef struct mca_coll_hcoll_component_t mca_coll_hcoll_component_t;
@@ -195,6 +209,7 @@ int mca_coll_hcoll_iallreduce(void *sbuf, void *rbuf, int count,
                             ompi_request_t** request,
                             mca_coll_base_module_t *module);
 int mca_coll_hcoll_progress(void);
+void mca_coll_hcoll_mem_release_cb(void *buf, size_t length, void *cbdata, bool from_alloc);
 END_C_DECLS
 
 #endif
