@@ -19,7 +19,7 @@
 #include "ompi_config.h"
 
 #include "ompi/constants.h"
-#include "ompi/datatype/datatype.h"
+#include "ompi/datatype/ompi_datatype.h"
 #include "coll_self.h"
 
 
@@ -33,12 +33,13 @@
 int mca_coll_self_allgather_intra(void *sbuf, int scount, 
                                   struct ompi_datatype_t *sdtype, void *rbuf, 
                                   int rcount, struct ompi_datatype_t *rdtype, 
-                                  struct ompi_communicator_t *comm)
+                                  struct ompi_communicator_t *comm,
+                                  mca_coll_base_module_t *module)
 {
     if (MPI_IN_PLACE == sbuf) {
         return MPI_SUCCESS;
     } else {
-        return ompi_ddt_sndrcv(sbuf, scount, sdtype,
+        return ompi_datatype_sndrcv(sbuf, scount, sdtype,
                                rbuf, rcount, rdtype);
     }
 }
