@@ -13,6 +13,8 @@
  * Copyright (c) 2007      Cisco Systems, Inc.  All rights reserved.
  * Copyright (c) 2012-2013 Los Alamos National Security, LLC.  All rights
  *                         reserved.
+ * Copyright (c) 2014      Research Organization for Information Science
+ *                         and Technology (RIST). All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -47,24 +49,30 @@ int MPI_Ialltoallv(const void *sendbuf, const int sendcounts[], const int sdispl
                    MPI_Request *request)
 {
     int i, size, err;
+    size_t sendtype_size, recvtype_size;
 
     MEMCHECKER(
         ptrdiff_t recv_ext;
         ptrdiff_t send_ext;
 
-        size = ompi_comm_remote_size(comm);
-        ompi_datatype_type_extent(recvtype, &recv_ext);
-        ompi_datatype_type_extent(sendtype, &send_ext);
-
-        memchecker_datatype(sendtype);
-        memchecker_datatype(recvtype);
         memchecker_comm(comm);
 
+        if (MPI_IN_PLACE != sendbuf) {
+            memchecker_datatype(sendtype);
+            ompi_datatype_type_extent(sendtype, &send_ext);
+        }
+
+        memchecker_datatype(recvtype);
+        ompi_datatype_type_extent(recvtype, &recv_ext);
+
+        size = OMPI_COMM_IS_INTER(comm)?ompi_comm_remote_size(comm):ompi_comm_size(comm);
         for ( i = 0; i < size; i++ ) {
-            /* check if send chunks are defined. */
-            memchecker_call(&opal_memchecker_base_isdefined,
-                            (char *)(sendbuf)+sdispls[i]*send_ext,
-                            sendcounts[i], sendtype);
+            if (MPI_IN_PLACE != sendbuf) {
+                /* check if send chunks are defined. */
+                memchecker_call(&opal_memchecker_base_isdefined,
+                                (char *)(sendbuf)+sdispls[i]*send_ext,
+                                sendcounts[i], sendtype);
+            }
             /* check if receive chunks are addressable. */
             memchecker_call(&opal_memchecker_base_isaddressable,
                             (char *)(recvbuf)+rdispls[i]*recv_ext,
@@ -89,21 +97,21 @@ int MPI_Ialltoallv(const void *sendbuf, const int sendcounts[], const int sdispl
             return OMPI_ERRHANDLER_INVOKE(comm, MPI_ERR_ARG, FUNC_NAME);
         }
 
-        /* We always define the remote group to be the same as the local
-           group in the case of an intracommunicator, so it's safe to
-           get the size of the remote group here for both intra- and
-           intercommunicators */
-
-        size = ompi_comm_remote_size(comm);
+        size = OMPI_COMM_IS_INTER(comm)?ompi_comm_remote_size(comm):ompi_comm_size(comm);
         for (i = 0; i < size; ++i) {
-            if (recvcounts[i] < 0) {
-                err = MPI_ERR_COUNT;
-            } else if (MPI_DATATYPE_NULL == recvtype || NULL == recvtype) {
-                err = MPI_ERR_TYPE;
-            } else {
-                OMPI_CHECK_DATATYPE_FOR_SEND(err, sendtype, sendcounts[i]);
-            }
+            OMPI_CHECK_DATATYPE_FOR_SEND(err, sendtype, sendcounts[i]);
             OMPI_ERRHANDLER_CHECK(err, comm, err, FUNC_NAME);
+            OMPI_CHECK_DATATYPE_FOR_RECV(err, recvtype, recvcounts[i]);
+            OMPI_ERRHANDLER_CHECK(err, comm, err, FUNC_NAME);
+        }
+
+        if (MPI_IN_PLACE != sendbuf && !OMPI_COMM_IS_INTER(comm)) {
+            int me = ompi_comm_rank(comm);
+            ompi_datatype_type_size(sendtype, &sendtype_size);
+            ompi_datatype_type_size(recvtype, &recvtype_size);
+            if ((sendtype_size*sendcounts[me]) != (recvtype_size*recvcounts[me])) {
+                return OMPI_ERRHANDLER_INVOKE(comm, MPI_ERR_TRUNCATE, FUNC_NAME);
+            }
         }
     }
 
