@@ -37,10 +37,6 @@ BEGIN_C_DECLS
 extern const char *ompi_info_type_ompi;
 extern const char *ompi_info_type_base;
 
-void ompi_info_show_ompi_version(const char *scope);
-
-void ompi_info_close_components(void);
-
 void ompi_info_do_config(bool want_all);
 
 END_C_DECLS
