@@ -10,9 +10,12 @@
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
- * Copyright (c) 2008-2013 Cisco Systems, Inc.  All rights reserved.
- * Copyright (c) 2012-2014 Los Alamos National Security, LLC. All rights
+ * Copyright (c) 2008-2015 Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2012-2015 Los Alamos National Security, LLC. All rights
  *                         reserved.
+ * Copyright (c) 2014      Intel, Inc. All rights reserved.
+ * Copyright (c) 2014      Research Organization for Information Science
+ *                         and Technology (RIST). All rights reserved.
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -327,6 +330,9 @@ static int mca_base_var_cache_files(bool rel_path_search)
                                  "Suppress warnings when attempting to set an overridden value (default: false)",
                                  MCA_BASE_VAR_TYPE_BOOL, NULL, 0, 0, OPAL_INFO_LVL_2,
                                  MCA_BASE_VAR_SCOPE_LOCAL, &mca_base_var_suppress_override_warning);
+    if (0 > ret) {
+        return ret;
+    }
 
     /* Aggregate MCA parameter files
      * A prefix search path to look up aggregate MCA parameter file
@@ -337,6 +343,9 @@ static int mca_base_var_cache_files(bool rel_path_search)
                                  "Aggregate MCA parameter file sets",
                                  MCA_BASE_VAR_TYPE_STRING, NULL, 0, 0, OPAL_INFO_LVL_3,
                                  MCA_BASE_VAR_SCOPE_READONLY, &mca_base_var_file_prefix);
+    if (0 > ret) {
+        return ret;
+    }
 
     ret = asprintf(&mca_base_param_file_path, "%s" OPAL_PATH_SEP "amca-param-sets%c%s",
                    opal_install_dirs.ompidatadir, OPAL_ENV_SEP, cwd);
@@ -423,7 +432,7 @@ int mca_base_var_get_value (int vari, const void *value,
     }
 
     if (!VAR_IS_VALID(var[0])) {
-        return OPAL_ERR_BAD_PARAM;
+        return OPAL_ERR_NOT_FOUND;
     }
 
     if (NULL != value) {
@@ -584,15 +593,20 @@ static int var_set_from_string (mca_base_var_t *var, char *src)
 
         if (MCA_BASE_VAR_TYPE_INT == var->mbv_type ||
             MCA_BASE_VAR_TYPE_UNSIGNED_INT == var->mbv_type) {
-            dst->intval = (int) int_value;
+            int *castme = (int*) var->mbv_storage;
+            *castme = int_value;
         } else if (MCA_BASE_VAR_TYPE_UNSIGNED_LONG == var->mbv_type) {
-            dst->ulval = (unsigned long) int_value;
+            unsigned long *castme = (unsigned long*) var->mbv_storage;
+            *castme = (unsigned long) int_value;
         } else if (MCA_BASE_VAR_TYPE_UNSIGNED_LONG_LONG == var->mbv_type) {
-            dst->ullval = (unsigned long long) int_value;
+            unsigned long long *castme = (unsigned long long*) var->mbv_storage;
+            *castme = (unsigned long long) int_value;
         } else if (MCA_BASE_VAR_TYPE_SIZE_T == var->mbv_type) {
-            dst->sizetval = (size_t) int_value;
+            size_t *castme = (size_t*) var->mbv_storage;
+            *castme = (size_t) int_value;
         } else if (MCA_BASE_VAR_TYPE_BOOL == var->mbv_type) {
-            dst->boolval = !!int_value;
+            bool *castme = (bool*) var->mbv_storage;
+            *castme = !!int_value;
         }
 
         return ret;
@@ -750,7 +764,7 @@ int mca_base_var_env_name(const char *param_name,
  */
 static int var_find_by_name (const char *full_name, int *vari, bool invalidok)
 {
-    mca_base_var_t *var;
+    mca_base_var_t *var = NULL;
     void *tmp;
     int rc;
 
@@ -762,7 +776,7 @@ static int var_find_by_name (const char *full_name, int *vari, bool invalidok)
 
     (void) var_get ((int)(uintptr_t) tmp, &var, false);
 
-    if (invalidok || VAR_IS_VALID(var[0])) {
+    if (invalidok || (var && VAR_IS_VALID(var[0]))) {
         *vari = (int)(uintptr_t) tmp;
         return OPAL_SUCCESS;
     }
@@ -834,7 +848,18 @@ int mca_base_var_set_flag (int vari, mca_base_var_flag_t flag, bool set)
  */
 int mca_base_var_get (int vari, const mca_base_var_t **var)
 {
-    return var_get (vari, (mca_base_var_t **) var, false);
+    int ret;
+    ret = var_get (vari, (mca_base_var_t **) var, false);
+
+    if (OPAL_SUCCESS != ret) {
+        return ret;
+    }
+
+    if (!VAR_IS_VALID(*(var[0]))) {
+        return OPAL_ERR_NOT_FOUND;
+    }
+
+    return OPAL_SUCCESS;
 }
 
 /*
@@ -973,6 +998,7 @@ int mca_base_var_finalize(void)
         if (NULL != mca_base_var_file_list) {
             opal_argv_free(mca_base_var_file_list);
         }
+        mca_base_var_file_list = NULL;
 
         (void) mca_base_var_group_finalize ();
         (void) mca_base_pvar_finalize ();
@@ -1084,15 +1110,31 @@ static int fixup_files(char **file_list, char * path, bool rel_path_search) {
 
 static int read_files(char *file_list, opal_list_t *file_values)
 {
-    int i, count;
+    char **tmp = opal_argv_split(file_list, OPAL_ENV_SEP);
+    int i, count, ret;
+
+    if (!tmp) {
+        return OPAL_ERR_OUT_OF_RESOURCE;
+    }
+
+    if (mca_base_var_file_list) {
+        count = opal_argv_count (mca_base_var_file_list);
+        ret = opal_argv_insert (&mca_base_var_file_list, count, tmp);
+        if (OPAL_SUCCESS != ret) {
+            return ret;
+        }
+
+        opal_argv_free (tmp);
+    } else {
+        mca_base_var_file_list = tmp;
+    }
+
+    count = opal_argv_count(mca_base_var_file_list);
 
     /* Iterate through all the files passed in -- read them in reverse
        order so that we preserve unix/shell path-like semantics (i.e.,
        the entries farthest to the left get precedence) */
 
-    mca_base_var_file_list = opal_argv_split(file_list, OPAL_ENV_SEP);
-    count = opal_argv_count(mca_base_var_file_list);
-
     for (i = count - 1; i >= 0; --i) {
         mca_base_parse_paramfile(mca_base_var_file_list[i], file_values);
     }
@@ -1116,6 +1158,42 @@ static int register_variable (const char *project_name, const char *framework_na
     /* Developer error. Storage can not be NULL and type must exist */
     assert (((flags & MCA_BASE_VAR_FLAG_SYNONYM) || NULL != storage) && type >= 0 && type < MCA_BASE_VAR_TYPE_MAX);
 
+#if OPAL_ENABLE_DEBUG
+    /* Developer error: check for alignments */
+    uintptr_t align = 0;
+    switch (type) {
+    case MCA_BASE_VAR_TYPE_INT:
+        align = OPAL_ALIGNMENT_INT;
+        break;
+    case MCA_BASE_VAR_TYPE_UNSIGNED_INT:
+        align = OPAL_ALIGNMENT_INT;
+        break;
+    case MCA_BASE_VAR_TYPE_UNSIGNED_LONG:
+        align = OPAL_ALIGNMENT_LONG;
+        break;
+    case MCA_BASE_VAR_TYPE_UNSIGNED_LONG_LONG:
+        align = OPAL_ALIGNMENT_LONG_LONG;
+        break;
+    case MCA_BASE_VAR_TYPE_SIZE_T:
+        align = OPAL_ALIGNMENT_SIZE_T;
+        break;
+    case MCA_BASE_VAR_TYPE_BOOL:
+        align = OPAL_ALIGNMENT_BOOL;
+        break;
+    case MCA_BASE_VAR_TYPE_DOUBLE:
+        align = OPAL_ALIGNMENT_DOUBLE;
+        break;
+    case MCA_BASE_VAR_TYPE_STRING:
+    default:
+        align = 0;
+        break;
+    }
+
+    if (0 != align) {
+        assert(((uintptr_t) storage) % align == 0);
+    }
+#endif
+
     /* There are data holes in the var struct */
     OPAL_DEBUG_ZERO(var);
 
@@ -1219,6 +1297,10 @@ static int register_variable (const char *project_name, const char *framework_na
             return OPAL_ERROR;
         }
 
+        if (!group->group_isvalid) {
+            group->group_isvalid = true;
+        }
+
         /* Verify the name components match */
         if (0 != compare_strings(framework_name, group->group_framework) ||
             0 != compare_strings(component_name, group->group_component) ||
@@ -1472,10 +1554,11 @@ static int var_set_from_file (mca_base_var_t *var, opal_list_t *file_values)
     const char *var_full_name = var->mbv_full_name;
     const char *var_long_name = var->mbv_long_name;
     bool deprecated = VAR_IS_DEPRECATED(var[0]);
+    bool is_synonym = VAR_IS_SYNONYM(var[0]);
     mca_base_var_file_value_t *fv;
     int ret;
 
-    if (VAR_IS_SYNONYM(var[0])) {
+    if (is_synonym) {
         ret = var_get (var->mbv_synonym_for, &var, true);
         if (OPAL_SUCCESS != ret) {
             return OPAL_ERROR;
@@ -1522,8 +1605,15 @@ static int var_set_from_file (mca_base_var_t *var, opal_list_t *file_values)
         }
 
         if (deprecated) {
+            const char *new_variable = "None (going away)";
+
+            if (is_synonym) {
+                new_variable = var->mbv_full_name;
+            }
+
             opal_show_help("help-mca-var.txt", "deprecated-mca-file",
-                           true, var_full_name, fv->mbvfv_file);
+                           true, var_full_name, fv->mbvfv_file,
+                           new_variable);
         }
 
         if (NULL != fv->mbvfv_file) {
@@ -1591,6 +1681,17 @@ static void var_constructor(mca_base_var_t *var)
  */
 static void var_destructor(mca_base_var_t *var)
 {
+    if (MCA_BASE_VAR_TYPE_STRING == var->mbv_type &&
+        NULL != var->mbv_storage &&
+        NULL != var->mbv_storage->stringval) {
+        free (var->mbv_storage->stringval);
+    }
+
+    /* don't release the boolean enumerator */
+    if (MCA_BASE_VAR_TYPE_BOOL != var->mbv_type && NULL != var->mbv_enumerator) {
+        OBJ_RELEASE(var->mbv_enumerator);
+    }
+
     if (NULL != var->mbv_variable_name) {
         free(var->mbv_variable_name);
     }
@@ -1603,16 +1704,6 @@ static void var_destructor(mca_base_var_t *var)
     if (NULL != var->mbv_description) {
         free(var->mbv_description);
     }
-    if (MCA_BASE_VAR_TYPE_STRING == var->mbv_type &&
-        NULL != var->mbv_storage &&
-        NULL != var->mbv_storage->stringval) {
-        free (var->mbv_storage->stringval);
-    }
-
-    /* don't release the boolean enumerator */
-    if (MCA_BASE_VAR_TYPE_BOOL != var->mbv_type && NULL != var->mbv_enumerator) {
-        OBJ_RELEASE(var->mbv_enumerator);
-    }
 
     /* Destroy the synonym array */
     OBJ_DESTRUCT(&var->mbv_synonyms);
@@ -1717,8 +1808,12 @@ static int var_value_string (mca_base_var_t *var, char **value_string)
             ret = var->mbv_enumerator->string_from_value(var->mbv_enumerator, value->intval, &tmp);
         }
 
+        if (OPAL_SUCCESS != ret) {
+            return ret;
+        }
+
         *value_string = strdup (tmp);
-        if (NULL == value_string) {
+        if (NULL == *value_string) {
             ret = OPAL_ERR_OUT_OF_RESOURCE;
         }
     }
@@ -1734,24 +1829,23 @@ int mca_base_var_check_exclusive (const char *project,
                                   const char *component_b,
                                   const char *param_b)
 {
-    mca_base_var_t *var_a, *var_b;
+    mca_base_var_t *var_a = NULL, *var_b = NULL;
     int var_ai, var_bi;
 
     /* XXX -- Remove me once the project name is in the componennt */
     project = NULL;
 
     var_ai = mca_base_var_find (project, type_a, component_a, param_a);
-    if (var_ai < 0) {
-        return OPAL_ERR_NOT_FOUND;
-    }
-
     var_bi = mca_base_var_find (project, type_b, component_b, param_b);
-    if (var_bi < 0) {
+    if (var_bi < 0 || var_ai < 0) {
         return OPAL_ERR_NOT_FOUND;
     }
 
     (void) var_get (var_ai, &var_a, true);
     (void) var_get (var_bi, &var_b, true);
+    if (NULL == var_a || NULL == var_b) {
+        return OPAL_ERR_NOT_FOUND;
+    }
 
     if (MCA_BASE_VAR_SOURCE_DEFAULT != var_a->mbv_source &&
         MCA_BASE_VAR_SOURCE_DEFAULT != var_b->mbv_source) {
@@ -1791,7 +1885,7 @@ int mca_base_var_dump(int vari, char ***out, mca_base_var_dump_type_t output_typ
     int i, line_count, line = 0, enum_count = 0;
     char *value_string, *source_string, *tmp;
     int synonym_count, ret, *synonyms = NULL;
-    mca_base_var_t *var, *original;
+    mca_base_var_t *var, *original=NULL;
     mca_base_var_group_t *group;
 
     ret = var_get(vari, &var, false);
@@ -1826,6 +1920,10 @@ int mca_base_var_dump(int vari, char ***out, mca_base_var_dump_type_t output_typ
     }
 
     source_string = source_name(var);
+    if (NULL == source_string) {
+        free (value_string);
+        return OPAL_ERR_OUT_OF_RESOURCE;
+    }
 
     if (MCA_BASE_VAR_DUMP_PARSABLE == output_type) {
         if (NULL != var->mbv_enumerator) {
@@ -1848,11 +1946,9 @@ int mca_base_var_dump(int vari, char ***out, mca_base_var_dump_type_t output_typ
 
         /* Output the value */
         asprintf(out[0] + line++, "%svalue:%s", tmp, value_string);
-        free(value_string);
 
         /* Output the source */
         asprintf(out[0] + line++, "%ssource:%s", tmp, source_string);
-        free(source_string);
 
         /* Output whether it's read only or writable */
         asprintf(out[0] + line++, "%sstatus:%s", tmp, VAR_IS_DEFAULT_ONLY(var[0]) ? "read-only" : "writeable");
@@ -1915,8 +2011,6 @@ int mca_base_var_dump(int vari, char ***out, mca_base_var_dump_type_t output_typ
                   VAR_IS_DEFAULT_ONLY(var[0]) ? "informational" : "parameter",
                   full_name, value_string, source_string, var->mbv_info_lvl + 1,
                   info_lvl_strings[var->mbv_info_lvl], var_type_names[var->mbv_type]);
-        free (value_string);
-        free (source_string);
 
         tmp = out[0][0];
         if (VAR_IS_DEPRECATED(var[0])) {
@@ -1978,10 +2072,10 @@ int mca_base_var_dump(int vari, char ***out, mca_base_var_dump_type_t output_typ
         }
 
         asprintf(out[0], "%s=%s (%s)", var->mbv_full_name, value_string, source_string);
-
-        free (value_string);
-        free (source_string);
     }
 
+    free (value_string);
+    free (source_string);
+
     return OPAL_SUCCESS;
 }
