@@ -43,16 +43,18 @@
 
 /*
  * Provide the hwloc object that corresponds to the given
- * LOGICAL processor id.  Remember: "processor" here [usually] means "core" --
+ * processor id of the given type.  Remember: "processor" here [usually] means "core" --
  * except that on some platforms, hwloc won't find any cores; it'll
  * only find PUs (!).  On such platforms, then do the same calculation
  * but with PUs instead of COREs.
  */
-hwloc_obj_t opal_hwloc_base_get_pu(hwloc_topology_t topo,  int lid)
+hwloc_obj_t opal_hwloc_base_get_pu(hwloc_topology_t topo,
+                                   int lid,
+                                   opal_hwloc_resource_type_t rtype)
 {
     hwloc_obj_type_t obj_type = HWLOC_OBJ_CORE;
     hwloc_obj_t obj;
-
+    
     /* hwloc isn't able to find cores on all platforms.  Example:
        PPC64 running RHEL 5.4 (linux kernel 2.6.18) only reports NUMA
        nodes and PU's.  Fine.  
@@ -70,14 +72,32 @@ hwloc_obj_t opal_hwloc_base_get_pu(hwloc_topology_t topo,  int lid)
         obj_type = HWLOC_OBJ_PU;
     }
 
+    if (OPAL_HWLOC_PHYSICAL == rtype) {
+        /* find the pu - note that we can only find physical PUs
+         * as cores do not have unique physical numbers (they are
+         * numbered within their sockets instead). So we find the
+         * specified PU, and then return the core object that contains it */
+        obj = hwloc_get_pu_obj_by_os_index(topo, lid);
+        OPAL_OUTPUT_VERBOSE((5, opal_hwloc_base_framework.framework_output,
+                             "physical cpu %d %s found in cpuset %s",
+                             lid, (NULL == obj) ? "not" : "is",
+                             (NULL == opal_hwloc_base_cpu_set) ? "None" : opal_hwloc_base_cpu_set));
+        /* we now need to shift upward to the core including this PU */
+        if (NULL != obj && HWLOC_OBJ_CORE == obj_type) {
+            obj = obj->parent;
+        }
+        return obj;
+    }
+            
+    opal_output_verbose(5, opal_hwloc_base_framework.framework_output,
+                        "Searching for %d LOGICAL PU", lid);
+    
     /* Now do the actual lookup. */
     obj = hwloc_get_obj_by_type(topo, obj_type, lid);
-    if (NULL == obj) {
-        opal_show_help("help-opal-hwloc-base.txt",
-                       "logical-cpu-not-found", true,
-                       opal_hwloc_base_cpu_set);
-        return NULL;
-    }
+    OPAL_OUTPUT_VERBOSE((5, opal_hwloc_base_framework.framework_output,
+                         "logical cpu %d %s found in cpuset %s",
+                         lid, (NULL == obj) ? "not" : "is",
+                         (NULL == opal_hwloc_base_cpu_set) ? "None" : opal_hwloc_base_cpu_set));
 
     /* Found the right core (or PU). Return the object */
     return obj;
@@ -91,6 +111,7 @@ int opal_hwloc_base_filter_cpus(hwloc_topology_t topo)
     hwloc_obj_t root, pu;
     hwloc_cpuset_t avail = NULL, pucpus, res;
     opal_hwloc_topo_data_t *sum;
+    opal_hwloc_obj_data_t *data;
     char **ranges=NULL, **range=NULL;
     int idx, cpu, start, end;
 
@@ -103,8 +124,6 @@ int opal_hwloc_base_filter_cpus(hwloc_topology_t topo)
 
     /* should only ever enter here once, but check anyway */
     if (NULL != sum->available) {
-        OPAL_OUTPUT_VERBOSE((5, opal_hwloc_base_framework.framework_output,
-                             "hwloc:base:filter_cpus specified - already done"));
         return OPAL_SUCCESS;
     }
 
@@ -130,33 +149,38 @@ int opal_hwloc_base_filter_cpus(hwloc_topology_t topo)
             case 1:
                 /* only one cpu given - get that object */
                 cpu = strtoul(range[0], NULL, 10);
-                if (NULL == (pu = opal_hwloc_base_get_pu(topo, cpu))) {
-                    opal_argv_free(ranges);
-                    opal_argv_free(range);
-                    return OPAL_ERROR;
+                if (NULL != (pu = opal_hwloc_base_get_pu(topo, cpu, OPAL_HWLOC_LOGICAL))) {
+                    hwloc_bitmap_and(pucpus, pu->online_cpuset, pu->allowed_cpuset);
+                    hwloc_bitmap_or(res, avail, pucpus);
+                    hwloc_bitmap_copy(avail, res);
+                    data = (opal_hwloc_obj_data_t*)pu->userdata;
+                    if (NULL == data) {
+                        pu->userdata = (void*)OBJ_NEW(opal_hwloc_obj_data_t);
+                        data = (opal_hwloc_obj_data_t*)pu->userdata;
+                    }
+                    data->npus++;
                 }
-                hwloc_bitmap_and(pucpus, pu->online_cpuset, pu->allowed_cpuset);
-                hwloc_bitmap_or(res, avail, pucpus);
-                hwloc_bitmap_copy(avail, res);
                 break;
             case 2:
                 /* range given */
                 start = strtoul(range[0], NULL, 10);
                 end = strtoul(range[1], NULL, 10);
                 for (cpu=start; cpu <= end; cpu++) {
-                    if (NULL == (pu = opal_hwloc_base_get_pu(topo, cpu))) {
-                        opal_argv_free(ranges);
-                        opal_argv_free(range);
-                        hwloc_bitmap_free(avail);
-                        return OPAL_ERROR;
+                    if (NULL != (pu = opal_hwloc_base_get_pu(topo, cpu, OPAL_HWLOC_LOGICAL))) {
+                        hwloc_bitmap_and(pucpus, pu->online_cpuset, pu->allowed_cpuset);
+                        hwloc_bitmap_or(res, avail, pucpus);
+                        hwloc_bitmap_copy(avail, res);
+                        data = (opal_hwloc_obj_data_t*)pu->userdata;
+                        if (NULL == data) {
+                            pu->userdata = (void*)OBJ_NEW(opal_hwloc_obj_data_t);
+                            data = (opal_hwloc_obj_data_t*)pu->userdata;
+                        }
+                        data->npus++;
                     }
-                    hwloc_bitmap_and(pucpus, pu->online_cpuset, pu->allowed_cpuset);
-                    hwloc_bitmap_or(res, avail, pucpus);
-                    hwloc_bitmap_copy(avail, res);
                 }
                 break;
             default:
-                return OPAL_ERR_BAD_PARAM;
+                break;
             }
             opal_argv_free(range);
         }
@@ -208,7 +232,7 @@ static void fill_cache_line_size(void)
 
 int opal_hwloc_base_get_topology(void)
 {
-    int rc;
+    int rc=OPAL_SUCCESS;
 
     OPAL_OUTPUT_VERBOSE((5, opal_hwloc_base_framework.framework_output,
                          "hwloc:base:get_topology"));
@@ -221,15 +245,11 @@ int opal_hwloc_base_get_topology(void)
             0 != hwloc_topology_load(opal_hwloc_topology)) {
             return OPAL_ERR_NOT_SUPPORTED;
         }
-
-        /* filter the cpus thru any default cpu set */
-        rc = opal_hwloc_base_filter_cpus(opal_hwloc_topology);
-        if (OPAL_SUCCESS != rc) {
+        if (OPAL_SUCCESS != (rc = opal_hwloc_base_filter_cpus(opal_hwloc_topology))) {
             return rc;
         }
     } else {
-        rc = opal_hwloc_base_set_topology(opal_hwloc_base_topo_file);
-        if (OPAL_SUCCESS != rc) {
+        if (OPAL_SUCCESS != (rc = opal_hwloc_base_set_topology(opal_hwloc_base_topo_file))) {
             return rc;
         }
     }
@@ -418,7 +438,7 @@ hwloc_cpuset_t opal_hwloc_base_get_available_cpus(hwloc_topology_t topo,
     opal_hwloc_topo_data_t *rdata;
     opal_hwloc_obj_data_t *data;
 
-    OPAL_OUTPUT_VERBOSE((5, opal_hwloc_base_framework.framework_output,
+    OPAL_OUTPUT_VERBOSE((10, opal_hwloc_base_framework.framework_output,
                          "hwloc:base: get available cpus"));
 
     /* get the node-level information */
@@ -431,8 +451,6 @@ hwloc_cpuset_t opal_hwloc_base_get_available_cpus(hwloc_topology_t topo,
         OPAL_OUTPUT_VERBOSE((5, opal_hwloc_base_framework.framework_output,
                              "hwloc:base:get_available_cpus first time - filtering cpus"));
     }
-    /* ensure the topo-level cpuset was prepared */
-    opal_hwloc_base_filter_cpus(topo);
 
     /* are we asking about the root object? */
     if (obj == root) {
@@ -481,9 +499,18 @@ hwloc_cpuset_t opal_hwloc_base_get_available_cpus(hwloc_topology_t topo,
 static void df_search_cores(hwloc_obj_t obj, unsigned int *cnt)
 {
     unsigned k;
+    opal_hwloc_obj_data_t *data;
 
     if (HWLOC_OBJ_CORE == obj->type) {
-        *cnt += 1;
+        data = (opal_hwloc_obj_data_t*)obj->userdata;
+        if (NULL == data) {
+            data = OBJ_NEW(opal_hwloc_obj_data_t);
+            obj->userdata = (void*)data;
+        }
+        if (NULL == opal_hwloc_base_cpu_set) {
+            data->npus = 1;
+        }
+        *cnt += data->npus;
         return;
     }
 
@@ -530,28 +557,8 @@ unsigned int opal_hwloc_base_get_npus(hwloc_topology_t topo,
     unsigned int cnt = 0;
     hwloc_cpuset_t cpuset;
 
-    /* if the object is a hwthread (i.e., HWLOC_OBJ_PU),
-     * then the answer is always 1 since there isn't
-     * anything underneath it
-     */
-    if (HWLOC_OBJ_PU == obj->type) {
-        return 1;
-    }
-
-    /* if the object is a core (i.e., HWLOC_OBJ_CORE) and
-     * we are NOT treating hwthreads as independent cpus,
-     * then the answer is also 1 since we don't allow
-     * you to use the underlying hwthreads as separate
-     * entities
-     */
-    if (HWLOC_OBJ_CORE == obj->type &&
-        !opal_hwloc_use_hwthreads_as_cpus) {
-        return 1;
-    }
-
     data = (opal_hwloc_obj_data_t*)obj->userdata;
-
-    if (NULL == data || 0 == data->npus) {
+    if (NULL == data || !data->npus_calculated) {
         if (!opal_hwloc_use_hwthreads_as_cpus) {
             /* if we are treating cores as cpus, then we really
              * want to know how many cores are in this object.
@@ -596,11 +603,13 @@ unsigned int opal_hwloc_base_get_npus(hwloc_topology_t topo,
             }
         }
         /* cache the info */
+        data = (opal_hwloc_obj_data_t*)obj->userdata;  // in case it was added
         if (NULL == data) {
             data = OBJ_NEW(opal_hwloc_obj_data_t);
             obj->userdata = (void*)data;
         }
         data->npus = cnt;
+        data->npus_calculated = true;
     }
 
     return data->npus;
@@ -859,6 +868,10 @@ static hwloc_obj_t df_search_min_bound(hwloc_topology_t topo,
     opal_hwloc_obj_data_t *data;
 
     if (target == start->type) {
+        /* only consider procs that are allowed */
+        if (0 == (k = opal_hwloc_base_get_npus(topo, start))) {
+            goto notfound;
+        }
         if (HWLOC_OBJ_CACHE == start->type && cache_level != start->attr->cache.depth) {
             goto notfound;
         }
@@ -868,6 +881,7 @@ static hwloc_obj_t df_search_min_bound(hwloc_topology_t topo,
             data = OBJ_NEW(opal_hwloc_obj_data_t);
             start->userdata = data;
         }
+            
         OPAL_OUTPUT_VERBOSE((5, opal_hwloc_base_framework.framework_output,
                              "hwloc:base:min_bound_under_obj object %s:%u nbound %u min %u",
                              hwloc_obj_type_string(target), start->logical_index,
@@ -1030,6 +1044,7 @@ void opal_hwloc_base_clear_usage(hwloc_topology_t topo)
 
 static int socket_to_cpu_set(char *cpus,
                              hwloc_topology_t topo,
+                             opal_hwloc_resource_type_t rtype,
                              hwloc_bitmap_t cpumask)
 {
     char **range;
@@ -1042,7 +1057,7 @@ static int socket_to_cpu_set(char *cpus,
     if ('*' == cpus[0]) {
         /* requesting cpumask for ALL sockets */
         obj = hwloc_get_root_obj(topo);
-        /* set to all available logical processors - essentially,
+        /* set to all available processors - essentially,
          * this specification equates to unbound
          */
         res = opal_hwloc_base_get_available_cpus(topo, obj);
@@ -1055,8 +1070,8 @@ static int socket_to_cpu_set(char *cpus,
     switch (range_cnt) {
     case 1:  /* no range was present, so just one socket given */
         socket_id = atoi(range[0]);
-        obj = opal_hwloc_base_get_obj_by_type(topo, HWLOC_OBJ_SOCKET, 0, socket_id, OPAL_HWLOC_LOGICAL);
-        /* get the available logical cpus for this socket */
+        obj = opal_hwloc_base_get_obj_by_type(topo, HWLOC_OBJ_SOCKET, 0, socket_id, rtype);
+        /* get the available cpus for this socket */
         res = opal_hwloc_base_get_available_cpus(topo, obj);
         hwloc_bitmap_or(cpumask, cpumask, res);
         break;
@@ -1066,8 +1081,8 @@ static int socket_to_cpu_set(char *cpus,
         upper_range = atoi(range[1]);
         /* cycle across the range of sockets */
         for (socket_id=lower_range; socket_id<=upper_range; socket_id++) {
-            obj = opal_hwloc_base_get_obj_by_type(topo, HWLOC_OBJ_SOCKET, 0, socket_id, OPAL_HWLOC_LOGICAL);
-            /* get the available logical cpus for this socket */
+            obj = opal_hwloc_base_get_obj_by_type(topo, HWLOC_OBJ_SOCKET, 0, socket_id, rtype);
+            /* get the available cpus for this socket */
             res = opal_hwloc_base_get_available_cpus(topo, obj);
             /* set the corresponding bits in the bitmask */
             hwloc_bitmap_or(cpumask, cpumask, res);
@@ -1084,6 +1099,7 @@ static int socket_to_cpu_set(char *cpus,
 
 static int socket_core_to_cpu_set(char *socket_core_list,
                                   hwloc_topology_t topo,
+                                  opal_hwloc_resource_type_t rtype,
                                   hwloc_bitmap_t cpumask)
 {
     int rc=OPAL_SUCCESS, i, j;
@@ -1102,7 +1118,7 @@ static int socket_core_to_cpu_set(char *socket_core_list,
     
     /* get the object for this socket id */
     if (NULL == (socket = opal_hwloc_base_get_obj_by_type(topo, HWLOC_OBJ_SOCKET, 0,
-                                                          socket_id, OPAL_HWLOC_LOGICAL))) {
+                                                          socket_id, rtype))) {
         opal_argv_free(socket_core);
         return OPAL_ERR_NOT_FOUND;
     }
@@ -1123,7 +1139,7 @@ static int socket_core_to_cpu_set(char *socket_core_list,
             corestr = socket_core[i];
         }
         if ('*' == corestr[0]) {
-            /* set to all available logical cpus on this socket */
+            /* set to all available cpus on this socket */
             res = opal_hwloc_base_get_available_cpus(topo, socket);
             hwloc_bitmap_or(cpumask, cpumask, res);
             /* we are done - already assigned all cores! */
@@ -1188,10 +1204,11 @@ static int socket_core_to_cpu_set(char *socket_core_list,
 
 int opal_hwloc_base_slot_list_parse(const char *slot_str,
                                     hwloc_topology_t topo,
+                                    opal_hwloc_resource_type_t rtype,
                                     hwloc_cpuset_t cpumask)
 {
-    char **item;
-    int rc, i, j;
+    char **item, **rngs;
+    int rc, i, j, k;
     hwloc_obj_t pu;
     hwloc_cpuset_t pucpus;
     char **range, **list;
@@ -1209,10 +1226,10 @@ int opal_hwloc_base_slot_list_parse(const char *slot_str,
     opal_output_verbose(5, opal_hwloc_base_framework.framework_output,
                         "slot assignment: slot_list == %s",
                         slot_str);
-    
+
     /* split at ';' */
     item = opal_argv_split(slot_str, ';');
-
+    
     /* start with a clean mask */
     hwloc_bitmap_zero(cpumask);
     /* loop across the items and accumulate the mask */
@@ -1231,75 +1248,95 @@ int opal_hwloc_base_slot_list_parse(const char *slot_str,
             if (NULL == strchr(item[i], ':')) {
                 /* binding just to the socket level, though
                  * it could specify multiple sockets
+                 * Skip the S and look for a ranges
                  */
-                if (OPAL_SUCCESS != (rc = socket_to_cpu_set(&item[i][1],  /* skip the 'S' */
-                                                            topo, cpumask))) {
-                    opal_argv_free(item);
-                    return rc;
+                rngs = opal_argv_split(&item[i][1], ',');
+                for (j=0; NULL != rngs[j]; j++) {
+                    if (OPAL_SUCCESS != (rc = socket_to_cpu_set(rngs[j], topo, rtype, cpumask))) {
+                        opal_argv_free(rngs);
+                        opal_argv_free(item);
+                        return rc;
+                    }
                 }
+                opal_argv_free(rngs);
             } else {
                 /* binding to a socket/whatever specification */
                 if ('S' == item[i][0] ||
                     's' == item[i][0]) {
-                    if (OPAL_SUCCESS != (rc = socket_core_to_cpu_set(&item[i][1],  /* skip the 'S' */
-                                                                     topo, cpumask))) {
-                        opal_argv_free(item);
-                        return rc;
+                    rngs = opal_argv_split(&item[i][1], ',');
+                    for (j=0; NULL != rngs[j]; j++) {
+                        if (OPAL_SUCCESS != (rc = socket_core_to_cpu_set(rngs[j], topo, rtype, cpumask))) {
+                            opal_argv_free(rngs);
+                            opal_argv_free(item);
+                            return rc;
+                        }
                     }
+                    opal_argv_free(rngs);
                 } else {
-                    if (OPAL_SUCCESS != (rc = socket_core_to_cpu_set(item[i],
-                                                                     topo, cpumask))) {
-                        opal_argv_free(item);
-                        return rc;
+                    rngs = opal_argv_split(item[i], ',');
+                    for (j=0; NULL != rngs[j]; j++) {
+                        if (OPAL_SUCCESS != (rc = socket_core_to_cpu_set(rngs[j], topo, rtype, cpumask))) {
+                            opal_argv_free(rngs);
+                            opal_argv_free(item);
+                            return rc;
+                        }
                     }
+                    opal_argv_free(rngs);
                 }
             }
         } else {
-            /* just a core specification - see if one or a range was given */
-            range = opal_argv_split(item[i], '-');
-            range_cnt = opal_argv_count(range);
-            /* see if a range was set or not */
-            switch (range_cnt) {
-            case 1:  /* only one core, or a list of cores, specified */
-                list = opal_argv_split(range[0], ',');
-                for (j=0; NULL != list[j]; j++) {
-                    core_id = atoi(list[j]);
-                    /* find the specified logical available cpu */
-                    if (NULL == (pu = opal_hwloc_base_get_pu(topo, core_id))) {
-                        opal_argv_free(range);
-                        opal_argv_free(item);
-                        return OPAL_ERROR;
+            rngs = opal_argv_split(item[i], ',');
+            for (k=0; NULL != rngs[k]; k++) {
+                /* just a core specification - see if one or a range was given */
+                range = opal_argv_split(rngs[k], '-');
+                range_cnt = opal_argv_count(range);
+                /* see if a range was set or not */
+                switch (range_cnt) {
+                case 1:  /* only one core, or a list of cores, specified */
+                    list = opal_argv_split(range[0], ',');
+                    for (j=0; NULL != list[j]; j++) {
+                        core_id = atoi(list[j]);
+                        /* find the specified available cpu */
+                        if (NULL == (pu = opal_hwloc_base_get_pu(topo, core_id, rtype))) {
+                            opal_argv_free(range);
+                            opal_argv_free(item);
+                            opal_argv_free(rngs);
+                            return OPAL_ERR_SILENT;
+                        }
+                        /* get the available cpus for that object */
+                        pucpus = opal_hwloc_base_get_available_cpus(topo, pu);
+                        /* set that in the mask */
+                        hwloc_bitmap_or(cpumask, cpumask, pucpus);
                     }
-                    /* get the available cpus for that object */
-                    pucpus = opal_hwloc_base_get_available_cpus(topo, pu);
-                    /* set that in the mask */
-                    hwloc_bitmap_or(cpumask, cpumask, pucpus);
-                }
-                opal_argv_free(list);
-                break;
+                    opal_argv_free(list);
+                    break;
                     
-            case 2:  /* range of core id's was given */
-                lower_range = atoi(range[0]);
-                upper_range = atoi(range[1]);
-                for (core_id=lower_range; core_id <= upper_range; core_id++) {
-                    /* find the specified logical available cpu */
-                    if (NULL == (pu = opal_hwloc_base_get_pu(topo, core_id))) {
-                        opal_argv_free(range);
-                        opal_argv_free(item);
-                        return OPAL_ERROR;
+                case 2:  /* range of core id's was given */
+                    lower_range = atoi(range[0]);
+                    upper_range = atoi(range[1]);
+                    for (core_id=lower_range; core_id <= upper_range; core_id++) {
+                        /* find the specified logical available cpu */
+                        if (NULL == (pu = opal_hwloc_base_get_pu(topo, core_id, rtype))) {
+                            opal_argv_free(range);
+                            opal_argv_free(item);
+                            opal_argv_free(rngs);
+                            return OPAL_ERR_SILENT;
+                        }
+                        /* get the available cpus for that object */
+                        pucpus = opal_hwloc_base_get_available_cpus(topo, pu);
+                        /* set that in the mask */
+                        hwloc_bitmap_or(cpumask, cpumask, pucpus);
                     }
-                    /* get the available cpus for that object */
-                    pucpus = opal_hwloc_base_get_available_cpus(topo, pu);
-                    /* set that in the mask */
-                    hwloc_bitmap_or(cpumask, cpumask, pucpus);
-                }
-                break;
+                    break;
                 
-            default:
-                opal_argv_free(range);
-                opal_argv_free(item);
-                return OPAL_ERROR;
+                default:
+                    opal_argv_free(range);
+                    opal_argv_free(item);
+                    opal_argv_free(rngs);
+                    return OPAL_ERROR;
+                }
             }
+            opal_argv_free(rngs);
         }
     }
     opal_argv_free(item);
@@ -1675,26 +1712,23 @@ static char *bitmap2rangestr(int bitmap)
 static int build_map(int *num_sockets_arg, int *num_cores_arg, 
                      hwloc_cpuset_t cpuset, int ***map, hwloc_topology_t topo)
 {
-    static int num_sockets = -1, num_cores = -1;
+    int num_sockets, num_cores;
     int socket_index, core_index, pu_index;
     hwloc_obj_t socket, core, pu;
     int **data;
 
-    /* Find out how many sockets we have (cached so that we don't have
-       to look this up every time) */
-    if (num_sockets < 0) {
-        num_sockets = hwloc_get_nbobjs_by_type(topo, HWLOC_OBJ_SOCKET);
-        /* some systems (like the iMac) only have one
-         * socket and so don't report a socket
-         */
-        if (0 == num_sockets) {
-            num_sockets = 1;
-        }
-        /* Lazy: take the total number of cores that we have in the
-           topology; that'll be more than the max number of cores
-           under any given socket */
-        num_cores = hwloc_get_nbobjs_by_type(topo, HWLOC_OBJ_CORE);
+    /* Find out how many sockets we have */
+    num_sockets = hwloc_get_nbobjs_by_type(topo, HWLOC_OBJ_SOCKET);
+    /* some systems (like the iMac) only have one
+     * socket and so don't report a socket
+     */
+    if (0 == num_sockets) {
+        num_sockets = 1;
     }
+    /* Lazy: take the total number of cores that we have in the
+       topology; that'll be more than the max number of cores
+       under any given socket */
+    num_cores = hwloc_get_nbobjs_by_type(topo, HWLOC_OBJ_CORE);
     *num_sockets_arg = num_sockets;
     *num_cores_arg = num_cores;
 
@@ -1763,7 +1797,7 @@ int opal_hwloc_base_cset2str(char *str, int len,
     int ret, socket_index, core_index;
     char tmp[BUFSIZ];
     const int stmp = sizeof(tmp) - 1;
-    int **map;
+    int **map=NULL;
     hwloc_obj_t root;
     opal_hwloc_topo_data_t *sum;
 
@@ -1790,7 +1824,6 @@ int opal_hwloc_base_cset2str(char *str, int len,
     if (OPAL_SUCCESS != (ret = build_map(&num_sockets, &num_cores, cpuset, &map, topo))) {
         return ret;
     }
-
     /* Iterate over the data matrix and build up the string */
     first = true;
     for (socket_index = 0; socket_index < num_sockets; ++socket_index) {
@@ -1808,8 +1841,12 @@ int opal_hwloc_base_cset2str(char *str, int len,
             }
         }
     }
-    free(map[0]);
-    free(map);
+    if (NULL != map) {
+        if (NULL != map[0]) {
+            free(map[0]);
+        }
+        free(map);
+    }
 
     return OPAL_SUCCESS;
 }
@@ -2054,3 +2091,37 @@ int opal_hwloc_get_sorted_numa_list(hwloc_topology_t topo, char* device_name, op
     }
     return OPAL_ERR_NOT_FOUND;
 }
+
+char* opal_hwloc_base_get_topo_signature(hwloc_topology_t topo)
+{
+    int nnuma, nsocket, nl3, nl2, nl1, ncore, nhwt;
+    char *sig=NULL, *arch=NULL;
+    hwloc_obj_t obj;
+    unsigned i;
+    
+    nnuma = opal_hwloc_base_get_nbobjs_by_type(topo, HWLOC_OBJ_NODE, 0, OPAL_HWLOC_AVAILABLE);
+    nsocket = opal_hwloc_base_get_nbobjs_by_type(topo, HWLOC_OBJ_SOCKET, 0, OPAL_HWLOC_AVAILABLE);
+    nl3 = opal_hwloc_base_get_nbobjs_by_type(topo, HWLOC_OBJ_CACHE, 3, OPAL_HWLOC_AVAILABLE);
+    nl2 = opal_hwloc_base_get_nbobjs_by_type(topo, HWLOC_OBJ_CACHE, 2, OPAL_HWLOC_AVAILABLE);
+    nl1 = opal_hwloc_base_get_nbobjs_by_type(topo, HWLOC_OBJ_CACHE, 1, OPAL_HWLOC_AVAILABLE);
+    ncore = opal_hwloc_base_get_nbobjs_by_type(topo, HWLOC_OBJ_CORE, 0, OPAL_HWLOC_AVAILABLE);
+    nhwt = opal_hwloc_base_get_nbobjs_by_type(topo, HWLOC_OBJ_PU, 0, OPAL_HWLOC_AVAILABLE);
+
+    /* get the root object so we can add the processor architecture */
+    obj = hwloc_get_root_obj(topo);
+    for (i=0; i < obj->infos_count; i++) {
+        if (0 == strcmp(obj->infos[i].name, "Architecture")) {
+            arch = obj->infos[i].value;
+            break;
+        }
+    }
+
+    if (NULL == arch) {
+        asprintf(&sig, "%dN:%dS:%dL3:%dL2:%dL1:%dC:%dH",
+                 nnuma, nsocket, nl3, nl2, nl1, ncore, nhwt);
+    } else {
+        asprintf(&sig, "%dN:%dS:%dL3:%dL2:%dL1:%dC:%dH:%s",
+                 nnuma, nsocket, nl3, nl2, nl1, ncore, nhwt, arch);
+    }
+    return sig;
+}
