@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2006      Los Alamos National Security, LLC.  All rights
+ * Copyright (c) 2006-2013 Los Alamos National Security, LLC.  All rights
  *                         reserved. 
  * Copyright (c) 2007-2010 Cisco Systems, Inc.  All rights reserved.
  * Copyright (c) 2010      Sandia National Laboratories. All rights reserved.
@@ -15,7 +15,7 @@
 #define OPAL_INSTALLDIRS_BASE_H
 
 #include "opal_config.h"
-
+#include "opal/mca/base/mca_base_framework.h"
 #include "opal/mca/installdirs/installdirs.h"
 
 /*
@@ -23,8 +23,10 @@
  */
 BEGIN_C_DECLS
 
-OPAL_DECLSPEC int opal_installdirs_base_open(void);
-OPAL_DECLSPEC int opal_installdirs_base_close(void);
+/**
+ * Framework structure declaration
+ */
+OPAL_DECLSPEC extern mca_base_framework_t opal_installdirs_base_framework;
 
 /* Just like opal_install_dirs_expand() (see installdirs.h), but will
    also insert the value of the environment variable $OPAL_DESTDIR, if
@@ -32,12 +34,6 @@ OPAL_DECLSPEC int opal_installdirs_base_close(void);
    setup routines of installdirs. */
 char * opal_install_dirs_expand_setup(const char* input);
 
-
-/*
- * Globals
- */
-OPAL_DECLSPEC extern opal_list_t opal_installdirs_components;
-
 END_C_DECLS
 
 #endif /* OPAL_BASE_INSTALLDIRS_H */
