@@ -183,7 +183,7 @@ static int orte_rmaps_base_register(mca_base_register_flag_t flags)
 
     rmaps_base_oversubscribe = false;
     (void) mca_base_var_register("orte", "rmaps", "base", "oversubscribe",
-                                 "If true, then allow oversubscription of nodes",
+                                 "If true, then allow oversubscription of nodes and overloading of processing elements",
                                  MCA_BASE_VAR_TYPE_BOOL, NULL, 0, 0,
                                  OPAL_INFO_LVL_9,
                                  MCA_BASE_VAR_SCOPE_READONLY, &rmaps_base_oversubscribe);
@@ -285,8 +285,8 @@ static int orte_rmaps_base_open(mca_base_open_flag_t flags)
     if (1 < orte_rmaps_base.cpus_per_rank) {
         orte_show_help("help-orte-rmaps-base.txt", "deprecated", true,
                        "--cpus-per-proc, -cpus-per-proc, --cpus-per-rank, -cpus-per-rank",
-                       "--map-by <obj>:PE=N",
-                       "rmaps_base_cpus_per_proc", "rmaps_base_mapping_policy=<obj>:PE=N");
+                       "--map-by <obj>:PE=N, default <obj>=NUMA",
+                       "rmaps_base_cpus_per_proc", "rmaps_base_mapping_policy=<obj>:PE=N, default <obj>=NUMA");
     }
 
     if (ORTE_SUCCESS != (rc = orte_rmaps_base_set_mapping_policy(&orte_rmaps_base.mapping,
@@ -386,14 +386,16 @@ static int orte_rmaps_base_open(mca_base_open_flag_t flags)
          */
         if (OPAL_BINDING_POLICY_IS_SET(opal_hwloc_binding_policy)) {
             if (opal_hwloc_use_hwthreads_as_cpus) {
-                if (OPAL_BIND_TO_HWTHREAD != OPAL_GET_BINDING_POLICY(opal_hwloc_binding_policy)) {
+                if (OPAL_BIND_TO_HWTHREAD != OPAL_GET_BINDING_POLICY(opal_hwloc_binding_policy) &&
+                    OPAL_BIND_TO_NONE != OPAL_GET_BINDING_POLICY(opal_hwloc_binding_policy)) {
                     orte_show_help("help-orte-rmaps-base.txt", "mismatch-binding", true,
                                    orte_rmaps_base.cpus_per_rank, "use-hwthreads-as-cpus",
                                    opal_hwloc_base_print_binding(opal_hwloc_binding_policy),
                                    "bind-to hwthread");
                     return ORTE_ERR_SILENT;
                 }
-            } else if (OPAL_BIND_TO_CORE != OPAL_GET_BINDING_POLICY(opal_hwloc_binding_policy)) {
+            } else if (OPAL_BIND_TO_CORE != OPAL_GET_BINDING_POLICY(opal_hwloc_binding_policy) &&
+                       OPAL_BIND_TO_NONE != OPAL_GET_BINDING_POLICY(opal_hwloc_binding_policy)) {
                 orte_show_help("help-orte-rmaps-base.txt", "mismatch-binding", true,
                                orte_rmaps_base.cpus_per_rank, "cores as cpus",
                                opal_hwloc_base_print_binding(opal_hwloc_binding_policy),
@@ -407,14 +409,26 @@ static int orte_rmaps_base_open(mca_base_open_flag_t flags)
                 OPAL_SET_BINDING_POLICY(opal_hwloc_binding_policy, OPAL_BIND_TO_CORE);
             }
         }
+        /* we also need to ensure we are mapping to a high-enough level to have
+         * multiple cpus beneath it - by default, we'll go to the NUMA level */
+        if (ORTE_MAPPING_GIVEN & ORTE_GET_MAPPING_DIRECTIVE(orte_rmaps_base.mapping)) {
+            if (ORTE_GET_MAPPING_POLICY(orte_rmaps_base.mapping) == ORTE_MAPPING_BYHWTHREAD ||
+              (ORTE_GET_MAPPING_POLICY(orte_rmaps_base.mapping) == ORTE_MAPPING_BYCORE &&
+              !opal_hwloc_use_hwthreads_as_cpus)) {
+                orte_show_help("help-orte-rmaps-base.txt", "mapping-too-low-init", true);
+                return ORTE_ERR_SILENT;
+            }
+        } else {
+            opal_output_verbose(5, orte_rmaps_base_framework.framework_output,
+                                "%s rmaps:base pe/rank set - setting mapping to BYNUMA",
+                                ORTE_NAME_PRINT(ORTE_PROC_MY_NAME));
+            ORTE_SET_MAPPING_POLICY(orte_rmaps_base.mapping, ORTE_MAPPING_BYNUMA);
+            ORTE_SET_MAPPING_DIRECTIVE(orte_rmaps_base.mapping, ORTE_MAPPING_GIVEN);
+        }
 #endif
     }
 
     if (orte_rmaps_base_pernode) {
-        orte_show_help("help-orte-rmaps-base.txt", "deprecated", true,
-                       "--pernode, -pernode", "--map-by ppr:1:node",
-                       "rmaps_base_pernode, rmaps_ppr_pernode",
-                       "rmaps_base_mapping_policy=ppr:1:node");
         /* there is no way to resolve this conflict, so if something else was
          * given, we have no choice but to error out
          */
@@ -431,10 +445,6 @@ static int orte_rmaps_base_open(mca_base_open_flag_t flags)
     }
 
     if (0 < orte_rmaps_base_n_pernode) {
-        orte_show_help("help-orte-rmaps-base.txt", "deprecated", true,
-                       "--npernode, -npernode", "--map-by ppr:N:node",
-                       "rmaps_base_n_pernode, rmaps_ppr_n_pernode",
-                       "rmaps_base_mapping_policy=ppr:N:node");
         /* there is no way to resolve this conflict, so if something else was
          * given, we have no choice but to error out
          */
@@ -451,10 +461,6 @@ static int orte_rmaps_base_open(mca_base_open_flag_t flags)
     }
 
     if (0 < orte_rmaps_base_n_persocket) {
-        orte_show_help("help-orte-rmaps-base.txt", "deprecated", true,
-                       "--npersocket, -npersocket", "--map-by ppr:N:socket",
-                       "rmaps_base_n_persocket, rmaps_ppr_n_persocket",
-                       "rmaps_base_mapping_policy=ppr:N:socket");
         /* there is no way to resolve this conflict, so if something else was
          * given, we have no choice but to error out
          */
@@ -499,6 +505,10 @@ static int orte_rmaps_base_open(mca_base_open_flag_t flags)
         }
         ORTE_UNSET_MAPPING_DIRECTIVE(orte_rmaps_base.mapping, ORTE_MAPPING_NO_OVERSUBSCRIBE);
         ORTE_SET_MAPPING_DIRECTIVE(orte_rmaps_base.mapping, ORTE_MAPPING_SUBSCRIBE_GIVEN);
+#if OPAL_HAVE_HWLOC
+        /* also set the overload allowed flag */
+        opal_hwloc_binding_policy |= OPAL_BIND_ALLOW_OVERLOAD;
+#endif
     }
 
     /* should we display a detailed (developer-quality) version of the map after determining it? */
@@ -543,6 +553,11 @@ static int check_modifiers(char *ck, orte_mapping_policy_t *tmp)
     int i;
     bool found = false;
 
+    opal_output_verbose(5, orte_rmaps_base_framework.framework_output,
+                        "%s rmaps:base check modifiers with %s",
+                        ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
+                        (NULL == ck) ? "NULL" : ck);
+
     if (NULL == ck) {
         return ORTE_SUCCESS;
     }
@@ -565,6 +580,10 @@ static int check_modifiers(char *ck, orte_mapping_policy_t *tmp)
                 return ORTE_ERR_SILENT;
             }
             orte_rmaps_base.cpus_per_rank = strtol(ptr, NULL, 10);
+            opal_output_verbose(5, orte_rmaps_base_framework.framework_output,
+                                "%s rmaps:base setting pe/rank to %d",
+                                ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
+                                orte_rmaps_base.cpus_per_rank);
             found = true;
         } else if (0 == strncasecmp(ck2[i], "oversubscribe", strlen(ck2[i]))) {
             ORTE_UNSET_MAPPING_DIRECTIVE(*tmp, ORTE_MAPPING_NO_OVERSUBSCRIBE);
@@ -603,6 +622,11 @@ int orte_rmaps_base_set_mapping_policy(orte_mapping_policy_t *policy,
     tmp = 0;
     *device = NULL;
 
+    opal_output_verbose(5, orte_rmaps_base_framework.framework_output,
+                        "%s rmaps:base set policy with %s",
+                        ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
+                        (NULL == inspec) ? "NULL" : inspec);
+
     if (NULL == inspec) {
         ORTE_SET_MAPPING_POLICY(tmp, ORTE_MAPPING_BYSOCKET);
     } else {
@@ -610,9 +634,28 @@ int orte_rmaps_base_set_mapping_policy(orte_mapping_policy_t *policy,
         /* see if a colon was included - if so, then we have a policy + modifier */
         ck = strchr(spec, ':');
         if (NULL != ck) {
+            /* if the colon is the first character of the string, then we
+             * just have modifiers on the default mapping policy */
+            if (ck == spec) {
+                ck++;
+                opal_output_verbose(5, orte_rmaps_base_framework.framework_output,
+                                    "%s rmaps:base only modifiers %s provided - assuming bysocket mapping",
+                                    ORTE_NAME_PRINT(ORTE_PROC_MY_NAME), ck);
+                ORTE_SET_MAPPING_POLICY(tmp, ORTE_MAPPING_BYSOCKET);
+                if (ORTE_ERR_SILENT == (rc = check_modifiers(ck, &tmp)) &&
+                    ORTE_ERR_BAD_PARAM != rc) {
+                    free(spec);
+                    return ORTE_ERR_SILENT;
+                }
+                free(spec);
+                goto setpolicy;
+            }
             /* split the string */
             *ck = '\0';
             ck++;
+            opal_output_verbose(5, orte_rmaps_base_framework.framework_output,
+                                "%s rmaps:base policy %s modifiers %s provided",
+                                ORTE_NAME_PRINT(ORTE_PROC_MY_NAME), spec, ck);
             /* if the policy is "dist", then we set the policy to that value
              * and save the second argument as the device
              */
@@ -688,6 +731,8 @@ int orte_rmaps_base_set_mapping_policy(orte_mapping_policy_t *policy,
             ORTE_SET_MAPPING_POLICY(tmp, ORTE_MAPPING_BYSLOT);
         } else if (0 == strncasecmp(spec, "node", len)) {
             ORTE_SET_MAPPING_POLICY(tmp, ORTE_MAPPING_BYNODE);
+        } else if (0 == strncasecmp(spec, "seq", len)) {
+            ORTE_SET_MAPPING_POLICY(tmp, ORTE_MAPPING_SEQ);
 #if OPAL_HAVE_HWLOC
         } else if (0 == strncasecmp(spec, "core", len)) {
             ORTE_SET_MAPPING_POLICY(tmp, ORTE_MAPPING_BYCORE);
@@ -719,9 +764,7 @@ int orte_rmaps_base_set_mapping_policy(orte_mapping_policy_t *policy,
         ORTE_SET_MAPPING_DIRECTIVE(tmp, ORTE_MAPPING_GIVEN);
     }
 
-#if OPAL_HAVE_HWLOC
  setpolicy:
-#endif
     *policy = tmp;
 
     return ORTE_SUCCESS;
