@@ -1,6 +1,8 @@
 /*
- * Copyright (c) 2013      Mellanox Technologies, Inc.
+ * Copyright (c) 2013-2015 Mellanox Technologies, Inc.
  *                         All rights reserved.
+ * Copyright (c) 2014      Research Organization for Information Science
+ *                         and Technology (RIST). All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -64,8 +66,8 @@ static int spml_ikrit_get_ep_address(spml_ikrit_mxm_ep_conn_info_t *ep_info,
                          (struct sockaddr *) &ep_info->addr.ptl_addr[ptlid],
                          &addrlen);
     if (MXM_OK != err) {
-        orte_show_help("help-spml-ikrit.txt",
-                       "unable to extract endpoint address",
+        orte_show_help("help-oshmem-spml-ikrit.txt",
+                       "unable to get endpoint address",
                        true,
                        mxm_error_string(err));
         return OSHMEM_ERROR;
@@ -339,17 +341,28 @@ OBJ_CLASS_INSTANCE( mxm_peer_t,
 
 int mca_spml_ikrit_del_procs(oshmem_proc_t** procs, size_t nprocs)
 {
-    size_t i;
-    opal_list_item_t *item;
+    size_t i, n;
+    int my_rank = oshmem_my_proc_id();
 
-    while (NULL != (item = opal_list_remove_first(&mca_spml_ikrit.active_peers))) {
+    oshmem_shmem_barrier();
+#if MXM_API >= MXM_VERSION(2,0)
+    if (mca_spml_ikrit.bulk_disconnect) {
+        mxm_ep_powerdown(mca_spml_ikrit.mxm_ep);
+    }
+#endif
+
+    while (NULL != opal_list_remove_first(&mca_spml_ikrit.active_peers)) {
     };
     OBJ_DESTRUCT(&mca_spml_ikrit.active_peers);
 
-    for (i = 0; i < nprocs; i++) {
+    for (n = 0; n < nprocs; n++) {
+        i = (my_rank + n) % nprocs;
         if (mca_spml_ikrit.mxm_peers[i]->mxm_conn) {
             mxm_ep_disconnect(mca_spml_ikrit.mxm_peers[i]->mxm_conn);
         }
+        if (mca_spml_ikrit.hw_rdma_channel && mca_spml_ikrit.mxm_peers[i]->mxm_hw_rdma_conn) {
+            mxm_ep_disconnect(mca_spml_ikrit.mxm_peers[i]->mxm_hw_rdma_conn);
+        }
         destroy_ptl_idx(i);
         if (mca_spml_ikrit.mxm_peers[i]) {
             OBJ_RELEASE(mca_spml_ikrit.mxm_peers[i]);
@@ -364,6 +377,7 @@ int mca_spml_ikrit_del_procs(oshmem_proc_t** procs, size_t nprocs)
 int mca_spml_ikrit_add_procs(oshmem_proc_t** procs, size_t nprocs)
 {
     spml_ikrit_mxm_ep_conn_info_t *ep_info = NULL;
+    spml_ikrit_mxm_ep_conn_info_t *ep_hw_rdma_info = NULL;
     spml_ikrit_mxm_ep_conn_info_t my_ep_info;
 #if MXM_API < MXM_VERSION(2,0)
     mxm_conn_req_t *conn_reqs;
@@ -372,7 +386,7 @@ int mca_spml_ikrit_add_procs(oshmem_proc_t** procs, size_t nprocs)
     size_t mxm_addr_len = MXM_MAX_ADDR_LEN;
 #endif
     mxm_error_t err;
-    size_t i;
+    size_t i, n;
     int rc = OSHMEM_ERROR;
     oshmem_proc_t *proc_self;
     int my_rank = oshmem_my_proc_id();
@@ -394,6 +408,15 @@ int mca_spml_ikrit_add_procs(oshmem_proc_t** procs, size_t nprocs)
     }
     memset(ep_info, 0x0, sizeof(spml_ikrit_mxm_ep_conn_info_t));
 
+    if (mca_spml_ikrit.hw_rdma_channel) {
+        ep_hw_rdma_info = malloc(nprocs * sizeof(spml_ikrit_mxm_ep_conn_info_t));
+        if (NULL == ep_hw_rdma_info) {
+            rc = OSHMEM_ERR_OUT_OF_RESOURCE;
+            goto bail;
+        }
+        memset(ep_hw_rdma_info, 0x0, sizeof(spml_ikrit_mxm_ep_conn_info_t));
+    }
+
     mca_spml_ikrit.mxm_peers = (mxm_peer_t **) malloc(nprocs
             * sizeof(*(mca_spml_ikrit.mxm_peers)));
     if (NULL == mca_spml_ikrit.mxm_peers) {
@@ -404,29 +427,45 @@ int mca_spml_ikrit_add_procs(oshmem_proc_t** procs, size_t nprocs)
 #if MXM_API < MXM_VERSION(2,0)
     if (OSHMEM_SUCCESS
             != spml_ikrit_get_ep_address(&my_ep_info, MXM_PTL_SELF)) {
-        return OSHMEM_ERROR;
+        rc = OSHMEM_ERROR;
+        goto bail;
     }
     if (OSHMEM_SUCCESS
             != spml_ikrit_get_ep_address(&my_ep_info, MXM_PTL_RDMA)) {
-        return OSHMEM_ERROR;
+        rc = OSHMEM_ERROR;
+        goto bail;
     }
 #else
+    if (mca_spml_ikrit.hw_rdma_channel) {
+        err = mxm_ep_get_address(mca_spml_ikrit.mxm_hw_rdma_ep, &my_ep_info.addr.ep_addr, &mxm_addr_len);
+        if (MXM_OK != err) {
+            orte_show_help("help-oshmem-spml-ikrit.txt", "unable to get endpoint address", true,
+                    mxm_error_string(err));
+            rc = OSHMEM_ERROR;
+            goto bail;
+        }
+        oshmem_shmem_allgather(&my_ep_info, ep_hw_rdma_info,
+                sizeof(spml_ikrit_mxm_ep_conn_info_t));
+    }
     err = mxm_ep_get_address(mca_spml_ikrit.mxm_ep, &my_ep_info.addr.ep_addr, &mxm_addr_len);
     if (MXM_OK != err) {
-        orte_show_help("help-shmem-spml-ikrit.txt", "unable to get endpoint address", true,
+        orte_show_help("help-oshmem-spml-ikrit.txt", "unable to get endpoint address", true,
                 mxm_error_string(err));
-        return OSHMEM_ERROR;
+        rc = OSHMEM_ERROR;
+        goto bail;
     }
 #endif
-
-    opal_progress_register(spml_ikrit_progress);
-
     oshmem_shmem_allgather(&my_ep_info, ep_info,
                            sizeof(spml_ikrit_mxm_ep_conn_info_t));
 
-    /* Get the EP connection requests for all the processes from modex */
-    for (i = 0; i < nprocs; ++i) {
+    opal_progress_register(spml_ikrit_progress);
 
+    /* Get the EP connection requests for all the processes from modex */
+    for (n = 0; n < nprocs; ++n) {
+
+        /* mxm 2.0 keeps its connections on a list. Make sure
+         * that list have different order on every rank */
+        i = (my_rank + n) % nprocs;
         mca_spml_ikrit.mxm_peers[i] = OBJ_NEW(mxm_peer_t);
         if (NULL == mca_spml_ikrit.mxm_peers[i]) {
             rc = OSHMEM_ERR_OUT_OF_RESOURCE;
@@ -449,6 +488,15 @@ int mca_spml_ikrit_add_procs(oshmem_proc_t** procs, size_t nprocs)
         if (OSHMEM_SUCCESS != create_ptl_idx(i))
                 goto bail;
         mxm_conn_ctx_set(mca_spml_ikrit.mxm_peers[i]->mxm_conn, mca_spml_ikrit.mxm_peers[i]);
+        if (mca_spml_ikrit.hw_rdma_channel) {
+            err = mxm_ep_connect(mca_spml_ikrit.mxm_hw_rdma_ep, ep_hw_rdma_info[i].addr.ep_addr, &mca_spml_ikrit.mxm_peers[i]->mxm_hw_rdma_conn);
+            if (MXM_OK != err) {
+                SPML_ERROR("MXM returned connect error: %s\n", mxm_error_string(err));
+                goto bail;
+            }
+        } else {
+            mca_spml_ikrit.mxm_peers[i]->mxm_hw_rdma_conn = mca_spml_ikrit.mxm_peers[i]->mxm_conn;
+        }
 #endif
     }
 
@@ -481,10 +529,18 @@ int mca_spml_ikrit_add_procs(oshmem_proc_t** procs, size_t nprocs)
         mxm_conn_ctx_set(conn_reqs[i].conn, mca_spml_ikrit.mxm_peers[i]);
     }
 
-    if (ep_info)
-        free(ep_info);
-    if (conn_reqs)
-        free(conn_reqs);
+    free(conn_reqs);
+#endif
+    free(ep_info);
+    free(ep_hw_rdma_info);
+
+    oshmem_shmem_barrier();
+#if MXM_API >= MXM_VERSION(2,0)
+    if (mca_spml_ikrit.bulk_connect) {
+        /* Need a barrier to ensure remote peers already created connection */
+        oshmem_shmem_barrier();
+        mxm_ep_wireup(mca_spml_ikrit.mxm_ep);
+    }
 #endif
 
     proc_self = oshmem_proc_group_find(oshmem_group_all, my_rank);
@@ -506,13 +562,12 @@ int mca_spml_ikrit_add_procs(oshmem_proc_t** procs, size_t nprocs)
     SPML_VERBOSE(50, "*** ADDED PROCS ***");
     return OSHMEM_SUCCESS;
 
-    bail:
-    #if MXM_API < MXM_VERSION(2,0)
-        if (conn_reqs)
-            free(conn_reqs);
-        if (ep_info)
-            free(ep_info);
-    #endif
+bail:
+#if MXM_API < MXM_VERSION(2,0)
+    free(conn_reqs);
+#endif
+    free(ep_info);
+    free(ep_hw_rdma_info);
     SPML_ERROR("add procs FAILED rc=%d", rc);
 
     return rc;
@@ -538,6 +593,7 @@ sshmem_mkey_t *mca_spml_ikrit_register(void* addr,
     }
 
     for (i = 0; i < MXM_PTL_LAST; i++) {
+        mkeys[i].u.key = MAP_SEGMENT_SHM_INVALID;
         switch (i) {
         case MXM_PTL_SHM:
             if ((int)shmid != MAP_SEGMENT_SHM_INVALID) {
@@ -562,7 +618,7 @@ sshmem_mkey_t *mca_spml_ikrit_register(void* addr,
 #if MXM_API < MXM_VERSION(2,0)
             mkeys[i].len = 0;
 #else
-            if (mca_spml_ikrit.ud_only) {
+            if (mca_spml_ikrit.ud_only && !mca_spml_ikrit.hw_rdma_channel) {
                 mkeys[i].len = 0;
                 break;
             }
@@ -589,10 +645,6 @@ sshmem_mkey_t *mca_spml_ikrit_register(void* addr,
             }
 #endif
             break;
-
-        default:
-            SPML_ERROR("unsupported PTL: %d", i);
-            goto error_out;
         }
         SPML_VERBOSE(5,
                      "rank %d ptl %d addr %p size %llu %s",
@@ -614,6 +666,7 @@ int mca_spml_ikrit_deregister(sshmem_mkey_t *mkeys)
 {
     int i;
 
+    MCA_SPML_CALL(fence());
     if (!mkeys)
         return OSHMEM_SUCCESS;
 
@@ -640,6 +693,8 @@ int mca_spml_ikrit_deregister(sshmem_mkey_t *mkeys)
             break;
         }
     }
+    free(mkeys);
+
     return OSHMEM_SUCCESS;
 
 }
@@ -672,17 +727,20 @@ int mca_spml_ikrit_oob_get_mkeys(int pe, uint32_t seg, sshmem_mkey_t *mkeys)
     if (seg > 1)
         return OSHMEM_ERROR;
 
+    mkeys[ptl].len = 0;
+    mkeys[ptl].u.key = MAP_SEGMENT_SHM_INVALID;
     return OSHMEM_SUCCESS;
 #else
     /* we are actually registering memory in 2.0 and later.
-     * So can not really skip mkey exchange
+     * So can only skip mkey exchange when ud is the only transport
      */
     if (mca_spml_ikrit.ud_only) {
         mkeys[ptl].len = 0;
-        mkeys[ptl].u.data = &mxm_empty_mem_key;
-    } else {
+        mkeys[ptl].u.key = MAP_SEGMENT_SHM_INVALID;
         return OSHMEM_SUCCESS;
-    }
+    } 
+
+    return OSHMEM_ERROR;
 #endif
 }
 
@@ -767,7 +825,7 @@ static inline int mca_spml_ikrit_get_shm(void *src_addr,
         return OSHMEM_ERROR;
     }
 
-    if (OPAL_UNLIKELY(!mca_memheap.memheap_is_symmetric_addr(src_addr) || src_addr == rva))
+    if (!mca_memheap_base_can_local_copy(r_mkey, src_addr))
         return OSHMEM_ERROR;
 
     SPML_VERBOSE(100,
@@ -1012,7 +1070,7 @@ static inline int mca_spml_ikrit_put_internal(void* dst_addr,
 #endif
     if (ptl_id == MXM_PTL_SHM) {
 
-        if (OPAL_LIKELY(mca_memheap.memheap_is_symmetric_addr(dst_addr) && dst_addr != rva)) {
+        if (mca_memheap_base_can_local_copy(r_mkey, dst_addr)) {
             memcpy((void *) (unsigned long) rva, src_addr, size);
             /* call progress as often as we would have with regular put */
             if (++count % SPML_IKRIT_PACKETS_PER_SYNC == 0)
@@ -1061,10 +1119,10 @@ static inline int mca_spml_ikrit_put_internal(void* dst_addr,
         put_req->mxm_req.base.flags = MXM_REQ_FLAG_SEND_LAZY|MXM_REQ_FLAG_SEND_SYNC;
     }
 #else
+    put_req->mxm_req.flags = 0;
     if (mca_spml_ikrit.free_list_max - mca_spml_ikrit.n_active_puts <= SPML_IKRIT_PUT_LOW_WATER ||
-            opal_list_get_size(&mca_spml_ikrit.active_peers) > mca_spml_ikrit.unsync_conn_max ||
+            (int)opal_list_get_size(&mca_spml_ikrit.active_peers) > mca_spml_ikrit.unsync_conn_max ||
             (mca_spml_ikrit.mxm_peers[dst]->n_active_puts + 1) % SPML_IKRIT_PACKETS_PER_SYNC == 0) {
-        put_req->mxm_req.flags = 0;
         need_progress = 1;
         put_req->mxm_req.opcode = MXM_REQ_OP_PUT_SYNC;
     } else  {
@@ -1154,7 +1212,7 @@ int mca_spml_ikrit_put_simple(void* dst_addr,
 #endif
     if (ptl_id == MXM_PTL_SHM) {
 
-        if (OPAL_LIKELY(mca_memheap.memheap_is_symmetric_addr(dst_addr) && dst_addr != rva)) {
+        if (mca_memheap_base_can_local_copy(r_mkey, dst_addr)) {
             memcpy((void *) (unsigned long) rva, src_addr, size);
             /* call progress as often as we would have with regular put */
             if (++count % SPML_IKRIT_PACKETS_PER_SYNC == 0)
@@ -1305,6 +1363,7 @@ int mca_spml_ikrit_fence(void)
 /* blocking receive */
 int mca_spml_ikrit_recv(void* buf, size_t size, int src)
 {
+    mxm_error_t ret = MXM_OK;
     mxm_recv_req_t req;
     char dummy_buf[1];
 
@@ -1328,9 +1387,12 @@ int mca_spml_ikrit_recv(void* buf, size_t size, int src)
     req.base.data.buffer.length = size == 0 ? sizeof(dummy_buf) : size;
     req.base.data.buffer.memh = NULL;
 
-    mxm_req_recv(&req);
+    ret = mxm_req_recv(&req);
+    if (MXM_OK != ret) {
+        return OSHMEM_ERROR;
+    }
     mca_spml_irkit_req_wait(&req.base);
-    if (req.base.error != MXM_OK) {
+    if (MXM_OK != req.base.error) {
         return OSHMEM_ERROR;
     }
     SPML_VERBOSE(100,
