@@ -18,6 +18,12 @@
 
 #include "oshmem/mca/memheap/memheap.h"
 
+#if OSHMEM_PROFILING
+#include "oshmem/include/pshmem.h"
+#pragma weak shfree = pshfree
+#include "oshmem/shmem/c/profile/defines.h"
+#endif
+
 void shfree(void* ptr)
 {
     int rc;
