@@ -13,6 +13,13 @@
 #include "oshmem/shmem/fortran/bindings.h"
 #include "oshmem/include/shmem.h"
 
+#if OSHMEM_PROFILING
+#include "oshmem/shmem/fortran/profile/pbindings.h"
+SHMEM_GENERATE_WEAK_BINDINGS(MY_PE, my_pe)
+#pragma weak _my_pe_ = p_my_pe_
+#include "oshmem/shmem/fortran/profile/defines.h"
+#endif
+
 SHMEM_GENERATE_FORTRAN_BINDINGS_FUNCTION (MPI_Fint,
         MY_PE,
         my_pe_,
@@ -21,6 +28,11 @@ SHMEM_GENERATE_FORTRAN_BINDINGS_FUNCTION (MPI_Fint,
         (void), 
         () )
 
+MPI_Fint _my_pe_(void)
+{
+    return my_pe_f();
+}
+
 MPI_Fint my_pe_f(void)
 {
     MPI_Fint rc;
